//---------------------------------------------------------------------
//
// FViewerView.cpp : implementation of the CFViewerView class
//
// This file contains the heart of the FontViewer utility. All display and
// print processing is done here, as well as interception of most
// UI functions, such as the vertical scroll bar and splitter bar, as
// well as event processing for the toolbar.
//
// Part of the FontViewer Utility, which first appeared
// in the May 26, 1998 issue of PC Magazine, U.S. Edition.
// Copyright (C) 1998 by Ziff-Davis, Inc. 
// Programmed by Jake Richter.
//
// The programs presented in PC Magazine can be copied but is copyright.
// You may make copies for others if no charge is involved, but copies
// or using any portion for any commerical purpose is strictly prohibited.
//
// For support, go to http://www.pcmag.com/discuss.htm and select 
// the Utilities area.
//
//---------------------------------------------------------------------

#include "stdafx.h"
#include "FViewer.h"
#include "FViewerDoc.h"
#include "MainFrm.h"
#include "FVToolBar.h"
#include "FViewerView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//---------------------------------------------------------------------
//
// Globals & Externals
//
//---------------------------------------------------------------------
static CFontList* pFaceList;     // Our font list pointer for EnumerateMyFonts

//---------------------------------------------------------------------
//
// Macros
//
// FNTFLG_TEST (x)   - Returns TRUE if flag x is set in m_fontFilter, 
//                     FALSE if not
// FNTFLG_SET (x, y) - Sets the flag x in m_fontFilter if y is TRUE, 
//                     else clears it
//
//---------------------------------------------------------------------
#define FNTFLG_TEST(x)     ((m_fontFilter & (x)) != 0)
#define FNTFLG_SET(x, y)   { m_fontFilter &= ~(x); m_fontFilter |= ((y) ? (x):0); }

//---------------------------------------------------------------------
//
// int CALLBACK EnumerateMyFonts
//
// This function is called as a result of the call to EnumFontFamiliesEx
// in CFViewerView::OnInitialUpdate(), once for every type face in the 
// system.
//
//---------------------------------------------------------------------
int CALLBACK EnumerateMyFonts(const LOGFONT *lpLogFont,
                              const TEXTMETRIC *lpMetrics,
                              unsigned long fontType, LPARAM lParam)
{
   int      faceFlag = 0;  // Initially, no flags set.
   LOGFONT  ourLogFont;    // Local copy of the LOGFONT structure.

   memcpy(&ourLogFont, lpLogFont, sizeof LOGFONT); // Make the copy

   //
   // We need to set our TRUETYPE, BITMAP, and NONANSI flags
   // based on the font type.
   //
   if (fontType == TRUETYPE_FONTTYPE)
      faceFlag |= FNT_TRUETYPE;
   else
      faceFlag |= FNT_BITMAP;

   if (ourLogFont.lfCharSet != ANSI_CHARSET)
      faceFlag |= FNT_NONANSI;
   else
      faceFlag |= FNT_ANSI;

   //
   // We also need to set the family type (we use it for filtering)
   //
   switch (ourLogFont.lfPitchAndFamily & 0xF0)
      {
      case FF_SWISS:
         faceFlag |= FNT_SANSSERIF;
         break;

      case FF_ROMAN:
         faceFlag |= FNT_SERIF;
         break;

      case FF_MODERN:
         faceFlag |= FNT_MODERN;
         break;

      case FF_SCRIPT:
         faceFlag |= FNT_SCRIPT;
         break;

      default:
         faceFlag |= FNT_OTHER;
         break;
      }

   //
   // Add the font to our font list.
   //
   pFaceList->NewFont(lpLogFont->lfFaceName, faceFlag, fontType, ourLogFont); // Otherwise, build table.

   return (1);
}

/////////////////////////////////////////////////////////////////////////////
// CFViewerView

IMPLEMENT_DYNCREATE(CFViewerView, CView)

BEGIN_MESSAGE_MAP(CFViewerView, CView)
   //{{AFX_MSG_MAP(CFViewerView)
   ON_COMMAND(ID_BUTTON_BITMAP, OnButtonBitmap)
   ON_UPDATE_COMMAND_UI(ID_BUTTON_BITMAP, OnUpdateButtonBitmap)
   ON_COMMAND(ID_BUTTON_BOLD, OnButtonBold)
   ON_UPDATE_COMMAND_UI(ID_BUTTON_BOLD, OnUpdateButtonBold)
   ON_COMMAND(ID_BUTTON_NONANSI, OnButtonNonANSI)
   ON_UPDATE_COMMAND_UI(ID_BUTTON_NONANSI, OnUpdateButtonNonANSI)
   ON_COMMAND(ID_BUTTON_ITALIC, OnButtonItalic)
   ON_UPDATE_COMMAND_UI(ID_BUTTON_ITALIC, OnUpdateButtonItalic)
   ON_COMMAND(ID_BUTTON_PLAIN, OnButtonPlain)
   ON_UPDATE_COMMAND_UI(ID_BUTTON_PLAIN, OnUpdateButtonPlain)
   ON_COMMAND(ID_BUTTON_TRUETYPE, OnButtonTruetype)
   ON_UPDATE_COMMAND_UI(ID_BUTTON_TRUETYPE, OnUpdateButtonTruetype)
   ON_COMMAND(ID_KYBD_DOWN, OnKybdDown)
   ON_COMMAND(ID_KYBD_END, OnKybdEnd)
   ON_COMMAND(ID_KYBD_HOME, OnKybdHome)
   ON_COMMAND(ID_KYBD_PGDN, OnKybdPgdn)
   ON_COMMAND(ID_KYBD_PGUP, OnKybdPgup)
   ON_COMMAND(ID_KYBD_UP, OnKybdUp)
   ON_WM_VSCROLL()
   ON_COMMAND(ID_FVSETTINGS, DoFVSetDlg)
   ON_CBN_SELCHANGE(ID_BUTTON_STRING, OnToolStringBoxList)
   ON_CBN_EDITCHANGE(ID_BUTTON_STRING, OnToolStringBoxEdit)
   ON_CBN_DROPDOWN(ID_BUTTON_STRING, OnToolStringBoxDropDown)
   ON_CBN_SETFOCUS(ID_BUTTON_STRING, OnToolStringBoxSetFocus)
   ON_CBN_SELCHANGE(ID_BUTTON_PTSIZE, OnToolPtSizeBoxList)
   ON_WM_MOUSEMOVE()
   ON_COMMAND(ID_BUTTON_MODERN, OnButtonModern)
   ON_UPDATE_COMMAND_UI(ID_BUTTON_MODERN, OnUpdateButtonModern)
   ON_COMMAND(ID_BUTTON_OTHER, OnButtonOther)
   ON_UPDATE_COMMAND_UI(ID_BUTTON_OTHER, OnUpdateButtonOther)
   ON_COMMAND(ID_BUTTON_SANSSERIF, OnButtonSansserif)
   ON_UPDATE_COMMAND_UI(ID_BUTTON_SANSSERIF, OnUpdateButtonSansserif)
   ON_UPDATE_COMMAND_UI(ID_BUTTON_SCRIPT, OnUpdateButtonScript)
   ON_COMMAND(ID_BUTTON_SCRIPT, OnButtonScript)
   ON_COMMAND(ID_BUTTON_SERIF, OnButtonSerif)
   ON_UPDATE_COMMAND_UI(ID_BUTTON_SERIF, OnUpdateButtonSerif)
   ON_COMMAND(ID_BUTTON_VIEWSELECT, OnButtonViewselect)
   ON_UPDATE_COMMAND_UI(ID_BUTTON_VIEWSELECT, OnUpdateButtonViewselect)
   ON_COMMAND(ID_BUTTON_ANSI, OnButtonAnsi)
   ON_UPDATE_COMMAND_UI(ID_BUTTON_ANSI, OnUpdateButtonAnsi)
   ON_WM_RBUTTONDOWN()
   ON_COMMAND(ID_SHOWCHARSET, OnShowCharSet)
   ON_COMMAND(ID_SHOWALPHANUM, OnShowAlphaNum)
   ON_WM_LBUTTONDOWN()
   ON_COMMAND(ID_VIEWSELECTED, OnViewSelected)
   ON_COMMAND(ID_BUTTON_BOLDITAL, OnButtonBoldItal)
   ON_UPDATE_COMMAND_UI(ID_BUTTON_BOLDITAL, OnUpdateButtonBoldItal)
   ON_WM_SETCURSOR( )
   ON_WM_KILLFOCUS()
   ON_UPDATE_COMMAND_UI(ID_VIEWSELECTED, OnUpdateViewSelected)
   ON_COMMAND(ID_KYBD_CTRLSPACE, OnKybdCtrlSpace)
   ON_COMMAND(ID_KYBD_SHIFTSPACE, OnKybdShiftSpace)
   ON_WM_CHAR()
   ON_COMMAND(ID_KYBD_F2, DoFVSetDlg)
	ON_WM_LBUTTONDBLCLK()
	//}}AFX_MSG_MAP

   ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------
//
// CFViewerView constructor
//
// Here we initialize a number of local variables.
//
//---------------------------------------------------------------------

CFViewerView::CFViewerView()
{
   m_DispArray.SetSize(50);// Assume at least 50 elements ultimately in array
   scrollPos = 0;          // Start at top
   scrollPageSize = 10;    // Number of lines of text in a scroll
   m_splitX = 300;         // Initial split position, 300 pixels
   m_splitTracking = FALSE;// Split tracking is off
   m_splitCursor = FALSE;  // No split cursor either.
                           // Load the horiz. double arrow cursor
   m_hCursorSizeWE = AfxGetApp()->LoadStandardCursor(IDC_SIZEWE);
   m_viewSelectMode = FALSE;  // Not in view selected mode initially.
   m_renderVariations = 0; // No render variations at first.

   return;
}

CFViewerView::~CFViewerView()
{
}

BOOL CFViewerView::PreCreateWindow(CREATESTRUCT& cs)
{
   // TODO: Modify the Window class or styles here by modifying
   //  the CREATESTRUCT cs

   return CView::PreCreateWindow(cs);
}

//---------------------------------------------------------------------
//
// PostNcDestroy()
//
// When we get this call, it means FontViewer is about to exit, in which case we
// want to dump data out to our INI file.
//
//---------------------------------------------------------------------
void CFViewerView::PostNcDestroy() 
{
   CWinApp* pApp = AfxGetApp();
   CString  section = INI_SECTION;
   CString  iniFile = pApp->m_pszProfileName;
   char     strValue[10];
   CMainFrame* mainWnd = (CMainFrame*)AfxGetMainWnd();
   WINDOWPLACEMENT	wndPlc;

   mainWnd->GetWindowPlacement(&wndPlc);

   _itoa(pointSizeTenths, strValue, 10); // Init the dialog values
   WritePrivateProfileString(section, "PointSizeTenths", strValue, iniFile);

   WritePrivateProfileString(section, "TestString", testString, iniFile);

   _itoa(FNTFLG_TEST(FNT_PLAIN), strValue, 10);
   WritePrivateProfileString(section, "Normal", strValue, iniFile);

   _itoa(FNTFLG_TEST(FNT_BOLD), strValue, 10);
   WritePrivateProfileString(section, "Bold", strValue, iniFile);

   _itoa(FNTFLG_TEST(FNT_ITALIC), strValue, 10);
   WritePrivateProfileString(section, "Italic", strValue, iniFile);

   _itoa(FNTFLG_TEST(FNT_BOLDITAL), strValue, 10);
   WritePrivateProfileString(section, "Bold-Italic", strValue, iniFile);

   _itoa(FNTFLG_TEST(FNT_TRUETYPE), strValue, 10);
   WritePrivateProfileString(section, "TrueType", strValue, iniFile);

   _itoa(FNTFLG_TEST(FNT_BITMAP), strValue, 10);
   WritePrivateProfileString(section, "Bitmap", strValue, iniFile);

   _itoa(FNTFLG_TEST(FNT_ANSI), strValue, 10);
   WritePrivateProfileString(section, "ANSI", strValue, iniFile);

   _itoa(FNTFLG_TEST(FNT_NONANSI), strValue, 10);
   WritePrivateProfileString(section, "NonANSI", strValue, iniFile);

   _itoa(FNTFLG_TEST(FNT_SANSSERIF), strValue, 10);
   WritePrivateProfileString(section, "SansSerif", strValue, iniFile);

   _itoa(FNTFLG_TEST(FNT_SERIF), strValue, 10);
   WritePrivateProfileString(section, "Serif", strValue, iniFile);

   _itoa(FNTFLG_TEST(FNT_MODERN), strValue, 10);
   WritePrivateProfileString(section, "FixedSpace", strValue, iniFile);

   _itoa(FNTFLG_TEST(FNT_SCRIPT), strValue, 10);
   WritePrivateProfileString(section, "Script", strValue, iniFile);

   _itoa(FNTFLG_TEST(FNT_OTHER), strValue, 10);
   WritePrivateProfileString(section, "Other", strValue, iniFile);

   _itoa(m_splitX, strValue, 10);
   WritePrivateProfileString(section, "Splitter", strValue, iniFile);

   //
   // Finally, we save the window position.
   //
   _itoa(wndPlc.showCmd, strValue, 10); // Save the show state.
   WritePrivateProfileString(section, "WinShow", strValue, iniFile);

   _itoa(wndPlc.flags, strValue, 10); // Save the windows flags
   WritePrivateProfileString(section, "WinFlag", strValue, iniFile);

   _itoa(wndPlc.rcNormalPosition.left, strValue, 10); // Set the default window position
   WritePrivateProfileString(section, "X1", strValue, iniFile);

   _itoa(wndPlc.rcNormalPosition.top, strValue, 10);
   WritePrivateProfileString(section, "Y1", strValue, iniFile);

   _itoa(wndPlc.rcNormalPosition.right, strValue, 10);
   WritePrivateProfileString(section, "X2", strValue, iniFile);

   _itoa(wndPlc.rcNormalPosition.bottom, strValue, 10);
   WritePrivateProfileString(section, "Y2", strValue, iniFile);

   CView::PostNcDestroy();
}


/////////////////////////////////////////////////////////////////////////////
// CFViewerView message handlers
//---------------------------------------------------------------------
//
// OnInitialUpdate()
//
// Here we do a little more initializations, as we're about ready to draw
// our initial view. The initialization done here generates our font list,
// and causes the font list to be parsed based on the initial set of 
// rendering flags. We also set up our vertical scroll bar here as well as
// load our last saved setting.
//
//---------------------------------------------------------------------

void CFViewerView::OnInitialUpdate() 
{
   LOGFONT        myLogFont;        // Our font structure
   HDC            myDC;             // Our device context
   CClientDC      dc(this);       // Get our device context
   CFViewerDoc*   pDoc = GetDocument();    // Pointer to our Document

   ASSERT_VALID(pDoc);

   CView::OnInitialUpdate();

   pFaceList = &(pDoc->faceList);   // Store the start of the font list
   myDC = dc.GetSafeHdc();       // Get the old style DC handle

   myLogFont.lfCharSet = DEFAULT_CHARSET; // Font search parms (find all)
   myLogFont.lfFaceName[0] = 0;
   myLogFont.lfPitchAndFamily = 0;

                              // Force the calls to EnumerateMyFonts
   EnumFontFamiliesEx(myDC, &myLogFont, &EnumerateMyFonts, NULL, 0);

   m_totalFonts = pFaceList->GetNumNodes(); // 1.09F

   //
   // This is where the render defaults are set, via the .INI read
   //  This mirrors the settings of defaults
   // in SetDefaults() in FVSetDlg.cpp
   //
   // Here we load our settings from the .INI file - all but the window settings.
   //
   CWinApp* pApp = AfxGetApp();
   CString  section = INI_SECTION;
   CString  iniFile = pApp->m_pszProfileName;
   char     strValue[60];
   CMainFrame* mainWnd = (CMainFrame*)AfxGetMainWnd();
   int      index;

   GetPrivateProfileString(section, "PointSizeTenths", "120",
                           strValue, 60, iniFile);
   pointSizeTenths   = atoi(strValue);
   GetPrivateProfileString(section, "TestString", "Test String - ABCDEFabcdef",
                           strValue, 60, iniFile);
   testString = strValue;

   GetPrivateProfileString(section, "Normal", "1", strValue, 60, iniFile);
   FNTFLG_SET(FNT_PLAIN, atoi(strValue));

   GetPrivateProfileString(section, "Bold", "0", strValue, 60, iniFile);
   FNTFLG_SET(FNT_BOLD, atoi(strValue));

   GetPrivateProfileString(section, "Italic", "0", strValue, 60, iniFile);
   FNTFLG_SET(FNT_ITALIC, atoi(strValue));

   GetPrivateProfileString(section, "Bold-Italic", "0", strValue, 60, iniFile);
   FNTFLG_SET(FNT_BOLDITAL, atoi(strValue));

   GetPrivateProfileString(section, "TrueType", "1", strValue, 60, iniFile);
   FNTFLG_SET(FNT_TRUETYPE, atoi(strValue));

   GetPrivateProfileString(section, "Bitmap", "1", strValue, 60, iniFile);
   FNTFLG_SET(FNT_BITMAP, atoi(strValue));

   GetPrivateProfileString(section, "ANSI", "1", strValue, 60, iniFile);
   FNTFLG_SET(FNT_ANSI, atoi(strValue));

   GetPrivateProfileString(section, "NonANSI", "0", strValue, 60, iniFile);
   FNTFLG_SET(FNT_NONANSI, atoi(strValue));

   GetPrivateProfileString(section, "SansSerif", "1", strValue, 60, iniFile);
   FNTFLG_SET(FNT_SANSSERIF, atoi(strValue));

   GetPrivateProfileString(section, "Serif", "1", strValue, 60, iniFile);
   FNTFLG_SET(FNT_SERIF, atoi(strValue));

   GetPrivateProfileString(section, "FixedSpace", "1", strValue, 60, iniFile);
   FNTFLG_SET(FNT_MODERN, atoi(strValue));

   GetPrivateProfileString(section, "Script", "1", strValue, 60, iniFile);
   FNTFLG_SET(FNT_SCRIPT, atoi(strValue));

   GetPrivateProfileString(section, "Other", "1", strValue, 60, iniFile);
   FNTFLG_SET(FNT_OTHER, atoi(strValue));

   GetPrivateProfileString(section, "Splitter", "300", strValue, 60, iniFile);
   m_splitX = atoi(strValue);         // Initial splitter position

   //
   // Because of our INI file, we also need to update the contents of the toolbar combo
   // and Point Size boxes to reflect stored settings.
   //
   _itoa(pointSizeTenths / 10, strValue, 10); // Init the dialog values
   index = mainWnd->m_wndToolBar.m_ptSizeBox.FindStringExact(0, strValue);
   if (index != CB_ERR)
        mainWnd->m_wndToolBar.m_ptSizeBox.SetCurSel(index);  // Set point size list box
   mainWnd->m_wndToolBar.m_stringBox.DeleteString(0);
   mainWnd->m_wndToolBar.m_stringBox.InsertString(0, testString);
   mainWnd->m_wndToolBar.m_stringBox.SetCurSel(0);
   mainWnd->m_wndToolBar.m_stringBoxStrings[0] = testString;

   FilterFonts();             // Using our current filters, filter the fonts

   return;
}

//---------------------------------------------------------------------
//
// FilterFonts
//
// This function is called whenever we need to reparse the font list we've
// built to see which fonts will get displayed. This function also sets the
// current maximum number of fonts to be displayed based on the given filter
// type.
//
//---------------------------------------------------------------------
void CFViewerView::FilterFonts()
{
   int            fontCounter = 0;  // Counter for # of fonts being rendered
   int            fontMask;         // Intermediate font mask
   int            renderVars = 0;   // Number of render variations
   int            dispFlag = FNT_DISPLAY; // Used to enable viewing of a font
   SCROLLINFO     myScrollInfo;           // Scrolling info
   CFViewerDoc*  pDoc = GetDocument();   // Point to our document class
   ASSERT_VALID(pDoc);
   CFontNode      *pNode = pDoc->faceList.GetStart(); // Start of face list

   m_rendCompound = 0;        // Clear the compounded style flag.

   if (!m_viewSelectMode)     // If not in view selected mode...
      {
      //
      // We need to see how many rendering variations are to be
      // displayed. There are four possible settings:
      // Normal, Bold, Italic, Bold-Italic
      //
      if (m_fontFilter & FNT_PLAIN)
         {
         m_rendVarTypes[renderVars] = FNT_PLAIN;
         m_rendCompound |= FNT_PLAIN;
         renderVars++;
         }
      if (m_fontFilter & FNT_BOLD)
         {
         m_rendVarTypes[renderVars] = FNT_BOLD;
         m_rendCompound |= FNT_BOLD;
         renderVars++;
         }
      if (m_fontFilter & FNT_ITALIC)
         {
         m_rendVarTypes[renderVars] = FNT_ITALIC;
         m_rendCompound |= FNT_ITALIC;
         renderVars++;
         }
      if (m_fontFilter & FNT_BOLDITAL)
         {
         m_rendVarTypes[renderVars] = FNT_BOLDITAL;
         m_rendCompound |= FNT_BOLDITAL;
         renderVars++;
         }

      m_renderVariations = renderVars; // Set number of styles for rendering

      if (renderVars == 0)
         dispFlag = 0;  // Disable display flag, since no font renderings selected.

      while (pNode != NULL)  // While not at end of list...
         {
         pNode->m_faceFlag &= ~FNT_DISPLAY;  // Clear the display flag for this face.

         //
         // For proper filtering, we need to look at our 6 possible flags.
         // (see FViewerView.h for a list)
         //
         // Next, check if any of the font flags match those that the user has set
         //
         if (fontMask = (m_fontFilter & pNode->m_faceFlag))
            {
            //
            // If there was a match of some sort, let's take a look at what
            // it was. If the NONANSI flag isn't set, and the font is NON-ANSI,
            // we shouldn't display it. Similarly, if the font is ANSI, 
            // and ANSI flag isn't set...
            // Also, if neither the TrueType or Bitmap flags are set, 
            // we should display
            // nothing.
            //
            if (fontMask & (FNT_TRUETYPE | FNT_BITMAP))
               {  // At least one of these two filters is set
               if (((m_fontFilter & FNT_NONANSI) && (pNode->m_faceFlag & FNT_NONANSI))
                  || ((m_fontFilter & FNT_ANSI) && (pNode->m_faceFlag & FNT_ANSI)))
                  {
                  //
                  // Finally, we need to filter based on the font family selection
                  // We do this by masking the fontMask from above with FNT_MSKFAMS,
                  // and if the result is non-0, we have a match of some sort.
                  //
                  if (fontMask & FNT_MSKFAMS)
                     {
                     pNode->m_faceFlag |= dispFlag;   // Set the display flag for font
                     // Increment fontCounter by count of face renderings
                     fontCounter += renderVars;
                     }
                  }
               }
            }
         pNode = pNode->GetNext();     // Get the next node in list.
         }
      }
   else
      {
      //
      // If we're in view selected mode, we don't bother with the normal 
      // font filtering. Instead, we run through each font node, and for 
      // every one that is "selected", we count how many styles of that 
      // font are selected and use that to build the totals needed below.
      //
      while (pNode != NULL)   // While not at end of list...
         {
         if (pNode->m_selected)
            {                 // If a font is selected, parse it.
            int   i;
            for (i = 0; i < m_renderVariations; i++)
               {
               if (pNode->m_selected & m_rendVarTypes[i])
                  fontCounter++; // Count each render variation selected
               }
            }
         pNode = pNode->GetNext();  // Go to next node
         }
      }

   maxFonts = fontCounter;    // Set the maxFonts - needed for scroll info
   maxScroll = maxFonts - 10; // Assume scroll size is 10.
   scrollPos = 0;             // Start at beginning of list again 
   m_lastSelected = NULL;     // No fonts currently selected
   m_curFontNode = NULL;      // No current


   CMainFrame* mainWnd = (CMainFrame*)AfxGetMainWnd();
   char  tmp[40];

   sprintf(tmp, "%d/%d", maxFonts, m_totalFonts * 4);
   mainWnd->m_wndStatusBar.SetPaneText(1, tmp, TRUE);


   //
   // Set scroll bar info
   //
   GetScrollInfo(SB_VERT, &myScrollInfo, SIF_ALL);
   myScrollInfo.nMin = 0;      // Set scrolling information
   myScrollInfo.nMax = maxFonts + 1;
   myScrollInfo.nPage = 10;
   myScrollInfo.nPos = 0;
   SetScrollInfo(SB_VERT, &myScrollInfo, TRUE);
   ShowScrollBar(SB_VERT, TRUE); // Display scroll bar

   m_curDispPos = DISP_TOP;	  // Sets first font as highlighted
   Invalidate();              // Force a redraw
   return;
}

//---------------------------------------------------------------------
//
// CString CharSetString(int charSet)
//
// This function returns a string describing the kind of font being displayed.
// The input value comes from the lfCharSet field in the LOGFONT structure.
//
//---------------------------------------------------------------------
CString  CFViewerView::CharSetString(int charSet)
{
   switch (charSet)
      {
      case ANSI_CHARSET:
         return("ANSI");
      case DEFAULT_CHARSET:
         return("Default");
      case SYMBOL_CHARSET:
         return("Symbol");
      case SHIFTJIS_CHARSET:
         return("ShiftJis");
      case HANGEUL_CHARSET:
         return("Hangeul");
      case GB2312_CHARSET:
         return("GB2312");
      case CHINESEBIG5_CHARSET:
         return("ChineseBig5");
      case OEM_CHARSET:
         return("OEM");
      case JOHAB_CHARSET:
         return("Johab");
      case HEBREW_CHARSET:
         return("Hebrew");
      case ARABIC_CHARSET:
         return("Arabic");
      case GREEK_CHARSET:
         return("Greek");
      case TURKISH_CHARSET:
         return("Turkish");
      case VIETNAMESE_CHARSET:
         return("Vietnamese");
      case THAI_CHARSET:
            return("Thai");
      case EASTEUROPE_CHARSET:
         return("EasternEuropean");
      case RUSSIAN_CHARSET:
         return("Russian");
      case MAC_CHARSET:
         return("AppleMac");
      case BALTIC_CHARSET:
         return("Baltic");
      default:
         return("Unknown");
      }
}

//---------------------------------------------------------------------
//
// OnDraw
//
// Here's where we update the display.
//
//---------------------------------------------------------------------

void CFViewerView::OnDraw(CDC* pDC)
{
   CRect rect;             // Client rectangle
   CFont *pMyFont;         // Pointer to our font information
   TEXTMETRIC myTextMet;   // Text Metrics (used for line spacing)
   SCROLLINFO myScroll;    // Scrolling info
   CFont *pOldFont;        // The system font
   int drawLine = 0;       // Current line to draw text on
   int curNodeNum = 0;     // The current node number
   int dispIndex = 0;      // Counter for the fonts being displayed
   CDisplayedFont dispNode;// Our displayed font information
   CString  fontName;      // Font Name String
   int   renderIndex;      // Index into the rendVarTypes array
   int   xSplit;           // The X boundary between the font and font name
   int   sysFntHgt;        // Height of the system font
   int drwFntHgt;          // Height of font we're drawing
   BOOL selected;          // Set in loop if current font rendering is tagged as selected
   CMainFrame* mainWnd = (CMainFrame*)AfxGetMainWnd(); // Pointer to main Window

   CFViewerDoc* pDoc = GetDocument();
   ASSERT_VALID(pDoc);
   CFontNode *pNode = pDoc->faceList.GetStart(); // Start of face list

   GetClientRect(rect);       // Let's get the dimensions of the client window

                              // Calculate the current page up/down amount.
   scrollPageSize = (rect.bottom * 10 / (pointSizeTenths + 40)) / 2;

   GetScrollInfo(SB_VERT, &myScroll, SIF_ALL);  // Load scroll info
   myScroll.nPage = scrollPageSize;    // Update scroll info (in case of key)
   myScroll.nPos = scrollPos;
   maxScroll = maxFonts - scrollPageSize;   // Just in case it's not set below.
   myScroll.nMax = maxFonts - 1;
   SetScrollInfo(SB_VERT, &myScroll, TRUE); // Store new? values
   pDC->SetTextAlign(TA_TOP | TA_LEFT);   // Set text display parameters
   pDC->SetTextColor(::GetSysColor(COLOR_WINDOWTEXT));
   pDC->SetBkMode(TRANSPARENT);
   pDC->GetTextMetrics(&myTextMet);       // Get font info on system font
   sysFntHgt = myTextMet.tmHeight;        // Set system font height

   pMyFont = new CFont;       // Create a new CFont for use below
   renderIndex = 0;
   xSplit = rect.left + m_splitX;

   m_fontHilit = FALSE;       // No font currently highlighted
   
   DrawVSplitter(m_splitX, pDC);   // Draw our splitter bar

   while (pNode != NULL)      // While not at end of list...
      {
      if (pNode->m_faceFlag & FNT_DISPLAY)   // If displayable font...
         {
         if (curNodeNum >= scrollPos) // While the proper fonts are displayable
            {
            //
            // If the current style bit is set in the m_selected field
            // in the font node, we have a "selected" font.
            // Note that if the font has not selections active, then
            // the compare will fail and the rendering properly be
            // set to be not selected.
            //
            if (pNode->m_selected & m_rendVarTypes[renderIndex])
               selected = TRUE;
            else
               selected = FALSE;

            //
            // If we have a selected font and we're in View Selected mode
            // or if we aren't in View Selected mode but the font has been
            // tagged for display (check made above), then we should
            // see about displaying the font.
            //
            if ((selected && m_viewSelectMode) || !m_viewSelectMode)
               {
               pNode->m_logFont.lfHeight = pointSizeTenths; // Set point size
               pNode->m_logFont.lfWidth = 0;             // Reset width
               pNode->m_logFont.lfQuality = PROOF_QUALITY;     // Set top quality

               //
               // Start building our desciptor string for current font.
               //
               fontName = pNode->GetString();
               fontName += " (";

               //
               // Here we render the different variations of our fonts.
               //
               switch (m_rendVarTypes[renderIndex])
                  {
                  case FNT_PLAIN:
                     pNode->m_logFont.lfWeight = FW_NORMAL;
                     pNode->m_logFont.lfItalic = FALSE;
                     fontName += "Normal";
                     break;
                  case FNT_BOLD:
                     pNode->m_logFont.lfWeight = FW_BOLD;
                     pNode->m_logFont.lfItalic = FALSE;
                     fontName += "Bold";
                     break;
                  case FNT_ITALIC:
                     pNode->m_logFont.lfWeight = FW_NORMAL;
                     pNode->m_logFont.lfItalic = TRUE;
                     fontName += "Italic";
                     break;
                  case FNT_BOLDITAL:
                     pNode->m_logFont.lfWeight = FW_BOLD;
                     pNode->m_logFont.lfItalic = TRUE;
                     fontName += "Bold-Italic";
                     break;
                  }

               //
               // Let's create the font rendering info here.
               //
               pMyFont->CreatePointFontIndirect(&(pNode->m_logFont), NULL);
               pOldFont = pDC->SelectObject(pMyFont);
               pDC->GetTextMetrics(&myTextMet);        // Get font info

               fontName += " - " + CharSetString(pNode->m_logFont.lfCharSet);

               //
               // And now, we close off the font descriptor by appending
               // information on whether it's a TrueType, Scalable
               // (non-TrueType like Adobe Type 1), or Bitmap.
               // In the case of a bitmap font, we also calculate the 
               // approximate point size (our formula is accurate down
               // to about 10 points) and display it.
               //
               if (!(myTextMet.tmPitchAndFamily & TMPF_VECTOR))
                  {     // If not vector, must be bitmap.
                  int size;
                  char tmp[10];

                  size = (myTextMet.tmHeight - myTextMet.tmInternalLeading) * 72;
                  size += pDC->GetDeviceCaps(LOGPIXELSY) - 1; // Round Up
                  size /= pDC->GetDeviceCaps(LOGPIXELSY);
                  fontName += " - ";
                  if (size <= 10)
                     {
                     tmp[0] = (char)177;  // +/- symbol
                     tmp[1] = '\0';    // End of string
                     fontName += tmp;
                     }
                  _itoa(size, tmp, 10);
                  fontName += tmp;
                  fontName += "pt Bitmap)";
                  }
               else
                  if (myTextMet.tmPitchAndFamily & TMPF_TRUETYPE)
                     fontName += " - TrueType)";
                  else
                     fontName += " - Scalable)";

               //
               // Next, lets draw our string in the font face, and the
               // descriptor we generated. We clip all font drawing to
               // make sure we don't make a mess on the screen.
               //
               drwFntHgt = myTextMet.tmHeight + 4;
               pDC->IntersectClipRect(xSplit + 5, drawLine,    // Clip font output
                              rect.right, drawLine + drwFntHgt + 2);
               pDC->TextOut(xSplit + 5, drawLine + 2, testString);  // Display string in typeface
               pDC->SelectClipRgn(NULL);     // Restore drawing to whole window
               pDC->SelectObject( pOldFont );// Switch back to old font
               pMyFont->DeleteObject();      // Free up GDI resource

                                          // Draw font name
               if (sysFntHgt < drwFntHgt)
                  {
                  pDC->IntersectClipRect(rect.left, drawLine,  // Clip font output
                              xSplit - 4, drawLine + drwFntHgt);
                  pDC->TextOut(2, drawLine + (drwFntHgt - sysFntHgt) / 2, fontName);
                  }
               else
                  {
                  pDC->IntersectClipRect(rect.left, drawLine,  // Clip font output
                              xSplit - 4, drawLine + sysFntHgt + 4);
                  pDC->TextOut(2, drawLine, fontName);
                  drwFntHgt = sysFntHgt + 4;
                  }
               pDC->SelectClipRgn(NULL);  // Restore drawing to whole window

               //
               // Let's then add the font displayed to our display node list,
               // which is used by ProcessDispFontPos(), and subsequently by the two
               // Show... dialogs and the font selection mechanism to figure
               // out the details of the displayed fonts.
               //
               dispNode.m_startY = drawLine;             // Starting Y position
               dispNode.m_endY = drawLine + drwFntHgt;   // Ending Y position
               dispNode.m_fontNode = pNode;              // Font node ptr
               dispNode.m_fullName = fontName;           // Full font name
               dispNode.m_styleFlag = m_rendVarTypes[renderIndex]; // Style

               //
               // Next, if we're not in View Selected mode, check if the
               // currently displayed font is selected, and if so, invert
               // the selection's box so it actually looks selected.
               //
               if (selected && !m_viewSelectMode)
                  {
                  CRect tmpRect;

                  tmpRect.top = dispNode.m_startY;
                  tmpRect.left = rect.left;
                  tmpRect.bottom = dispNode.m_endY;
                  tmpRect.right = rect.right;
                  pDC->InvertRect(tmpRect);     // Make the selection
                  }

               dispNode.m_selected = selected;  // Set the selection field
               m_DispArray.SetAtGrow(dispIndex, dispNode);  // Add display node
               dispIndex++;                     // Up to next array entry
               drawLine += drwFntHgt;           // Update vertical drawing pos
               }
            }

         if (m_viewSelectMode)   // Need to muck with curNodeNum here
            {
            //
            // We need to increment curNodeNum for every selected 
            // font/style that qualifies for display, even if it 
            // didn't get displayed.
            //
            if (pNode->m_selected & m_rendVarTypes[renderIndex])
               curNodeNum++;
            }
         else
            curNodeNum++;
         }

      //
      // Cycle through the render variations before moving on to the next node
      //
      if (++renderIndex >= m_renderVariations)
         {
         pNode = pNode->GetNext();     // Go to next node of typeface info
         renderIndex = 0;
         }
      if (drawLine >= rect.bottom)     // Stop bothering to draw if not vis.
		   {
         maxScroll = (maxFonts - dispIndex) + 4;  // Set maximum scroll position.
         break;
		   }
      }

   m_curMinY = m_curMaxY = -1;   // Y values reinitialized
   dispNode.m_startY = 29999;    // Set terminators
   dispNode.m_endY = 29999;      // Set terminators
   m_DispArray.SetAtGrow(dispIndex, dispNode);  // Set terminator node

   delete pMyFont;               // Delete our resource

   CPoint point;
   point.x = 0;					// Has to be in client area.

   if (m_curDispPos == DISP_TOP || !dispIndex)
	   {
	   point.y = 0;
	   ProcessDispFontPos(point, pDC);	// Set highlight
      m_curDispPos = DISP_TOP;
	   }

   if (m_curDispPos == DISP_BOT)
	   {
	   point.y = m_DispArray[dispIndex - 1].m_endY - 1;	// Make sure whole highlight visible
	   if (point.y > rect.bottom)
		   point.y = m_DispArray[dispIndex - 2].m_startY + 1;
	   ProcessDispFontPos(point, pDC);	// Set highlight
	   }
   if (m_curDispPos == DISP_SAME)
	   {
	   if (m_curDispIndex > dispIndex - 1)			// If at end of display, make sure we're not refer
		   m_curDispIndex = dispIndex - 1;
	   point.y = m_DispArray[m_curDispIndex].m_startY + 1;
	   ProcessDispFontPos(point, pDC);	// Set highlight
	   }

   m_curDispPos = DISP_SAME;

   return;
}

//---------------------------------------------------------------------
//
// OnPreparePrinting()
//
// This function is called by MFC to let us know we're being asked
// to print out what we're displaying.
//
//---------------------------------------------------------------------
BOOL CFViewerView::OnPreparePrinting(CPrintInfo* pInfo) 
{
   CRect       rect;          // Client rectangle

   GetClientRect(rect); // Let's get the dimensions of the client window

   m_printLineNum = 0;  // Start with line 0
   m_printPageNum = 1;  // Start with page 1

   return CView::DoPreparePrinting(pInfo);   // Put up print dialog box
}

//---------------------------------------------------------------------
//
// OnPrepareDC()
//
// This function is called by MFC before every page to verify whether
// we have any pages left to print.
//
//---------------------------------------------------------------------
void CFViewerView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
   CView::OnPrepareDC(pDC, pInfo);

   if (pInfo)  // Only if pInfo is valid (if NULL, it's for screen, not printer)
      {
      if (m_printPageNum != -1)  // If OnPrint's printed all, we're done.
         pInfo->m_bContinuePrinting = TRUE;  // Still more to print
      else
         pInfo->m_bContinuePrinting = FALSE; // Done
      }

   return;
}

//---------------------------------------------------------------------
//
// OnPrint()
//
// This function is the printer analogue to OnDraw() - functionality
// is virtually identical expect that we print headers and footers,
// and we don't display bitmap font sizes, since Windows kindly substitutes
// scalable fonts for bitmap ones when we print.
//
//---------------------------------------------------------------------
void CFViewerView::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
   CRect    rect;          // Client rectangle
   CFont    *pMyFont;      // Pointer to our font information
   CFont    *pSysFont;     // Pointer to the system font we'll be using
   TEXTMETRIC myTextMet;   // Text Metrics (used for line spacing)
   CFont    *pOldFont;     // The system font
   int      drawLine;      // Current line to draw text on
   int      curNodeNum = 0;// The current node number
   CString  fontName;      // Font Name String
   int      renderIndex;   // Index into the rendVarTypes array
   int      xSplit;        // The X boundary between the font and font name
   int      sysFntHgt;     // Height of the system font
   int      drwFntHgt;     // Height of font we're drawing
   BOOL     selected;      // Set in loop if current font rendering is tagged as selected
   CString  header;        // Header string & tmp string
   char     tmp[10];       // Used for _itoa
   int      xPerInch, yPerInch;  // Logical pixels per inch, needed for placement.

   CFViewerDoc* pDoc = GetDocument();
   ASSERT_VALID(pDoc);
   CFontNode *pNode = pDoc->faceList.GetStart();   // Start of face list

   xPerInch = pDC->GetDeviceCaps(LOGPIXELSX);      // Get printer DPI
   yPerInch = pDC->GetDeviceCaps(LOGPIXELSY);

   rect = pInfo->m_rectDraw;  // Let's get the dimensions of the client window

   pMyFont = new CFont;       // Create new CFonts for use below
   pSysFont = new CFont;

   pDC->SetTextColor(RGB(0,0,0));   // Black print
   pDC->SetBkMode(TRANSPARENT);

   pSysFont->CreatePointFont(80, "Arial", pDC);   // Use a 8 point System font
   pOldFont = pDC->SelectObject(pSysFont);         //   for header/footer

   pDC->GetTextMetrics(&myTextMet);        // Get font info on system font

   //
   // Now we need to make a nice header & footer.
   //
   header = "FontViewer - Page ";
   _itoa(m_printPageNum, tmp, 10);
   header += tmp;
   pDC->SetTextAlign(TA_BOTTOM | TA_CENTER);  // Set footer display parameters
                                    // Print footer (name, page number)
   pDC->TextOut((rect.right - rect.left) / 2, rect.bottom - (yPerInch / 2), header);

   pDC->SetTextAlign(TA_TOP | TA_CENTER);  // Set header display parameters

   // Next, let's make a string of our options:

   if (m_viewSelectMode)
      header = "Viewing Selected: ";
   else
      header = "Complete Set: ";

   _itoa(pointSizeTenths / 10, tmp, 10);

   header += tmp;
   header += "pt, ";

   if (m_fontFilter & FNT_TRUETYPE)
      header += "TrueType";
   if (m_fontFilter & FNT_BITMAP)
      header += "Bitmap/Raster";
   if (m_fontFilter & FNT_ANSI)
      header += ", ANSI";
   if (m_fontFilter & FNT_NONANSI)
      header += ", Non-ANSI";
   if (m_fontFilter & FNT_PLAIN)
      header += ", Normal";
   if (m_fontFilter & FNT_BOLD)
      header += ", Bold";
   if (m_fontFilter & FNT_ITALIC)
      header += ", Italic";
   if (m_fontFilter & FNT_BOLDITAL)
      header += ", Bold-Italic";
   if (m_fontFilter & FNT_SANSSERIF)
      header += ", Sans Serif";
   if (m_fontFilter & FNT_SERIF)
      header += ", Serif";
   if (m_fontFilter & FNT_MODERN)
      header += ", Fixed Space";
   if (m_fontFilter & FNT_SCRIPT)
      header += ", Script";
   if (m_fontFilter & FNT_OTHER)
      header += ", Decorative/Other";
                                 // Print description at top
   pDC->TextOut((rect.right - rect.left) / 2, yPerInch / 2, header);

   //
   // Now we're back to our regular font display/printing.
   //
   pDC->SetTextAlign(TA_TOP | TA_LEFT);  // Set text display parameters for rest of page

   pDC->SelectObject( pOldFont );          // Switch back to old font
   pSysFont->DeleteObject();           // Free up GDI resource

   pSysFont->CreatePointFont(100, "Arial", pDC);  // Use a 10 point System font names
   pDC->SelectObject(pSysFont);

   pDC->GetTextMetrics(&myTextMet);        // Get font info on system font

   sysFntHgt = myTextMet.tmHeight;        // Set system font height

   renderIndex = 0;

   rect.left += (xPerInch * 3) / 4; // Trim .75 inches on left & right, 1 top & bot
   rect.top += yPerInch;
   rect.right -= (xPerInch * 3) / 4;
   rect.bottom -= yPerInch;

   drawLine = rect.top; // Start the initial drawing line

   xSplit = rect.left + 3 * xPerInch;

   while (pNode != NULL)  // While not at end of list...
      {
      if (pNode->m_faceFlag & FNT_DISPLAY)   // If displayable font...
         {
         if (curNodeNum >= m_printLineNum) // While the proper fonts are displayable
            {
            //
            // If the current style bit is set in the m_selected field
            // in the font node, we have a "selected" font.
            // Note that if the font has not selections active, then
            // the compare will fail and the rendering properly be
            // set to be not selected.
            //
            if (pNode->m_selected & m_rendVarTypes[renderIndex])
               selected = TRUE;
            else
               selected = FALSE;

               if ((selected && m_viewSelectMode) || !m_viewSelectMode)
               {
                           // Select the font into the device context
               pNode->m_logFont.lfHeight = pointSizeTenths; // Set point size
               pNode->m_logFont.lfWidth = 0;             // Reset width
               pNode->m_logFont.lfQuality = PROOF_QUALITY;     // Set top quality

               fontName = pNode->GetString();
               fontName += " (";

               //
               // Here we render the different variations of our fonts.
               //
               switch (m_rendVarTypes[renderIndex])
                  {
                  case FNT_PLAIN:
                     pNode->m_logFont.lfWeight = FW_NORMAL;
                     pNode->m_logFont.lfItalic = FALSE;
                     fontName += "Normal";
                     break;
                  case FNT_BOLD:
                     pNode->m_logFont.lfWeight = FW_BOLD;
                     pNode->m_logFont.lfItalic = FALSE;
                     fontName += "Bold";
                     break;
                  case FNT_ITALIC:
                     pNode->m_logFont.lfWeight = FW_NORMAL;
                     pNode->m_logFont.lfItalic = TRUE;
                     fontName += "Italic";
                     break;
                  case FNT_BOLDITAL:
                     pNode->m_logFont.lfWeight = FW_BOLD;
                     pNode->m_logFont.lfItalic = TRUE;
                     fontName += "Bold-Italic";
                     break;
                  }

               pMyFont->CreatePointFontIndirect(&(pNode->m_logFont), pDC);
               pOldFont = pDC->SelectObject(pMyFont);
               pDC->GetTextMetrics(&myTextMet);    // Get font info

               fontName += " - " + CharSetString(pNode->m_logFont.lfCharSet);

               if (!(myTextMet.tmPitchAndFamily & TMPF_VECTOR))
                  fontName += "- Bitmap)";
               else
                  if (myTextMet.tmPitchAndFamily & TMPF_TRUETYPE)
                     fontName += " - TrueType)";
                  else
                     fontName += " - Scalable)";

               drwFntHgt = myTextMet.tmHeight + LOGY_PTS(2);
               if (drawLine + drwFntHgt >= rect.bottom) // Stop bothering to draw if not vis.
                    break;
               pDC->IntersectClipRect(xSplit + LOGX_PTS(10), drawLine,  // Clip font output
                              rect.right, drawLine + drwFntHgt);
               pDC->TextOut(xSplit + LOGX_PTS(10), drawLine + LOGY_PTS(2), testString);  // Display string in typeface
               pDC->SelectClipRgn(NULL);           // Restore drawing to whole window
               pDC->SelectObject(pSysFont);        // Switch back to system font
               pMyFont->DeleteObject();            // Free up GDI resource

                                             // Draw font name
               if (sysFntHgt < drwFntHgt)
                  {
                  pDC->IntersectClipRect(rect.left, drawLine,     // Clip font output
                              xSplit - LOGX_PTS(10), drawLine + drwFntHgt);
                  pDC->TextOut(rect.left, drawLine + (drwFntHgt - sysFntHgt) / 2, fontName);
                  }
               else
                  {
                  pDC->IntersectClipRect(rect.left, drawLine,     // Clip font output
                              xSplit - LOGX_PTS(10), 
                              drawLine + sysFntHgt + LOGY_PTS(2));
                  pDC->TextOut(rect.left, drawLine, fontName);
                  drwFntHgt = sysFntHgt + LOGY_PTS(2);
                  }
               pDC->SelectClipRgn(NULL);           // Restore drawing to whole window
               drawLine += drwFntHgt;              // Update vertical drawing pos
               }
            }
         if (m_viewSelectMode)   // Need to muck with curNodeNum here
            {
            //
            // We need to increment curNodeNum for every selected font/style that 
            // qualifies for display, even if it didn't get displayed.
            //
            if (pNode->m_selected & m_rendVarTypes[renderIndex])
               curNodeNum++;
            }
         else
            curNodeNum++;
         }

      if (++renderIndex >= m_renderVariations)
         {
         pNode = pNode->GetNext();     // Go to next node of typeface info
         renderIndex = 0;
         }
      }

   m_printLineNum += (curNodeNum - m_printLineNum);   // Add number of lines on this page
   if (pNode)        // If not at end of list...
      m_printPageNum++;
   else
      m_printPageNum = -1; // End of print

   pDC->SelectObject( pOldFont );          // Switch back to old font
   pSysFont->DeleteObject();           // Free up GDI resource

   delete pSysFont;
   delete pMyFont;
   return;
}


//---------------------------------------------------------------------
//
// The following four functions are used to post common message box
// errors. These are also called from FViewerView.cpp when similar
// toolbar button related problems occur. They are located in this class
// so that they are correctly modal.
//
//---------------------------------------------------------------------
void  CFViewerView::ErrBadStyleVariation()
{
   MessageBox("At least one style variation,\nNormal, Bold, Italic, or Bold-Italic,\nmust be enabled.",
            "Style Variation Selection Error",
            MB_ICONEXCLAMATION | MB_OK);
   return;
}

void  CFViewerView::ErrBadFontType()
{
   MessageBox("At least one font type,\nTrueType or Bitmap/Raster,\nmust be enabled.",
            "Font Type Selection Error",
            MB_ICONEXCLAMATION | MB_OK);
   return;
}

void  CFViewerView::ErrBadCharacterSet()
{
   MessageBox("At least one character set,\nANSI or Non-ANSI,\nmust be enabled.",
            "Character Set Selection Error",
            MB_ICONEXCLAMATION | MB_OK);
   return;
}

void  CFViewerView::ErrBadAttributeFilter()
{
   MessageBox("At least one attribute filter,\nSans Serif, Serif, etc.,\nmust be enabled.",
            "Attribute Filter Selection Error",
            MB_ICONEXCLAMATION | MB_OK);
   return;
}

//---------------------------------------------------------------------
//
// The following functions handle the toolbar buttons when pressed,
// as well as to set the status of the buttons. There's also validation
// performed to make sure that at least one button in each category is
// pressed.
//
//---------------------------------------------------------------------
void CFViewerView::OnButtonBitmap() 
{
   m_fontFilter ^= FNT_BITMAP;

   if (!(m_fontFilter & FNT_MSKCLASS))
      {  // If we get here, we don't have anything pressed in this category
      m_fontFilter |= FNT_BITMAP;
      ErrBadFontType();
      }
   else
      {
      m_viewSelectMode = FALSE;  // Only switch off VS mode if changed.
      FilterFonts();
      }
   return;
}

void CFViewerView::OnUpdateButtonBitmap(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(FNTFLG_TEST(FNT_BITMAP));
   return;
}

void CFViewerView::OnButtonBold() 
{
   m_fontFilter ^= FNT_BOLD;
   if (!(m_fontFilter & FNT_MSKFACES))
      {  // If we get here, we don't have anything pressed in this category
      m_fontFilter |= FNT_BOLD;
      ErrBadStyleVariation();
      }
   else
      {
      m_viewSelectMode = FALSE;  // Only switch off VS mode if changed.
      FilterFonts();
      }
   return;
}

void CFViewerView::OnUpdateButtonBold(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(FNTFLG_TEST(FNT_BOLD));
   return;
}


void CFViewerView::OnButtonBoldItal() 
{
   m_fontFilter ^= FNT_BOLDITAL;
   if (!(m_fontFilter & FNT_MSKFACES))
      {  // If we get here, we don't have anything pressed in this category
      m_fontFilter |= FNT_BOLDITAL;
      ErrBadStyleVariation();
      }
   else
      {
      m_viewSelectMode = FALSE;  // Only switch off VS mode if changed.
      FilterFonts();
      }
   return;
}

void CFViewerView::OnUpdateButtonBoldItal(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(FNTFLG_TEST(FNT_BOLDITAL));
   return;
}

void CFViewerView::OnButtonAnsi() 
{
   m_fontFilter ^= FNT_ANSI;
   if (!(m_fontFilter & FNT_MSKSETS))
      {  // If we get here, we don't have anything pressed in this category
      m_fontFilter |= FNT_ANSI;
      ErrBadCharacterSet();
      }
   else
      {
      m_viewSelectMode = FALSE;  // Only switch off VS mode if changed.
      FilterFonts();
      }
   return;
}

void CFViewerView::OnUpdateButtonAnsi(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(FNTFLG_TEST(FNT_ANSI));
   return;
}

void CFViewerView::OnButtonNonANSI() 
{
   m_fontFilter ^= FNT_NONANSI;
   if (!(m_fontFilter & FNT_MSKSETS))
      {  // If we get here, we don't have anything pressed in this category
      m_fontFilter |= FNT_NONANSI;
      ErrBadCharacterSet();
      }
   else
      {
      m_viewSelectMode = FALSE;  // Only switch off VS mode if changed.
      FilterFonts();
      }
   return;
}

void CFViewerView::OnUpdateButtonNonANSI(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(FNTFLG_TEST(FNT_NONANSI));
   return;
}

void CFViewerView::OnButtonItalic() 
{
   m_fontFilter ^= FNT_ITALIC;
   if (!(m_fontFilter & FNT_MSKFACES))
      {  // If we get here, we don't have anything pressed in this category
      m_fontFilter |= FNT_ITALIC;
      ErrBadStyleVariation();
      }
   else
      {
      m_viewSelectMode = FALSE;  // Only switch off VS mode if changed.
      FilterFonts();
      }
   return;
}

void CFViewerView::OnUpdateButtonItalic(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(FNTFLG_TEST(FNT_ITALIC));
   return;
}

void CFViewerView::OnButtonPlain() 
{
   m_fontFilter ^= FNT_PLAIN;
   if (!(m_fontFilter & FNT_MSKFACES))
      {  // If we get here, we don't have anything pressed in this category
      m_fontFilter |= FNT_PLAIN;
      ErrBadStyleVariation();
      }
   else
      {
      m_viewSelectMode = FALSE;  // Only switch off VS mode if changed.
      FilterFonts();
      }
   return;
}

void CFViewerView::OnUpdateButtonPlain(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(FNTFLG_TEST(FNT_PLAIN));
   return;
}

void CFViewerView::OnButtonTruetype() 
{
   m_fontFilter ^= FNT_TRUETYPE;
   if (!(m_fontFilter & FNT_MSKCLASS))
      {  // If we get here, we don't have anything pressed in this category
      m_fontFilter |= FNT_TRUETYPE;
      ErrBadFontType();
      }
   else
      {
      m_viewSelectMode = FALSE;  // Only switch off VS mode if changed.
      FilterFonts();
      }
   return;
}

void CFViewerView::OnUpdateButtonTruetype(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(FNTFLG_TEST(FNT_TRUETYPE));
   return;
}


void CFViewerView::OnButtonModern() 
{
   m_fontFilter ^= FNT_MODERN;
   if (!(m_fontFilter & FNT_MSKFAMS))
      {  // If we get here, we don't have anything pressed in this category
      m_fontFilter |= FNT_MODERN;
      ErrBadAttributeFilter();
      }
   else
      {
      m_viewSelectMode = FALSE;  // Only switch off VS mode if changed.
      FilterFonts();
      }
   return;
}

void CFViewerView::OnUpdateButtonModern(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(FNTFLG_TEST(FNT_MODERN));
   return;
}

void CFViewerView::OnButtonOther() 
{
   m_fontFilter ^= FNT_OTHER;
   if (!(m_fontFilter & FNT_MSKFAMS))
      {  // If we get here, we don't have anything pressed in this category
      m_fontFilter |= FNT_OTHER;
      ErrBadAttributeFilter();
      }
   else
      {
      m_viewSelectMode = FALSE;  // Only switch off VS mode if changed.
      FilterFonts();
      }
   return;
}

void CFViewerView::OnUpdateButtonOther(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(FNTFLG_TEST(FNT_OTHER));
   return;
}

void CFViewerView::OnButtonSansserif() 
{
   m_fontFilter ^= FNT_SANSSERIF;
   if (!(m_fontFilter & FNT_MSKFAMS))
      {  // If we get here, we don't have anything pressed in this category
      m_fontFilter |= FNT_SANSSERIF;
      ErrBadAttributeFilter();
      }
   else
      {
      m_viewSelectMode = FALSE;  // Only switch off VS mode if changed.
      FilterFonts();
      }
   return;
}

void CFViewerView::OnUpdateButtonSansserif(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(FNTFLG_TEST(FNT_SANSSERIF));
   return;
}

void CFViewerView::OnButtonScript() 
{
   m_fontFilter ^= FNT_SCRIPT;
   if (!(m_fontFilter & FNT_MSKFAMS))
      {  // If we get here, we don't have anything pressed in this category
      m_fontFilter |= FNT_SCRIPT;
      ErrBadAttributeFilter();
      }
   else
      {
      m_viewSelectMode = FALSE;  // Only switch off VS mode if changed.
      FilterFonts();
      }
   return;
}

void CFViewerView::OnUpdateButtonScript(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(FNTFLG_TEST(FNT_SCRIPT));
   return;
}

void CFViewerView::OnButtonSerif() 
{
   m_fontFilter ^= FNT_SERIF;
   if (!(m_fontFilter & FNT_MSKFAMS))
      {  // If we get here, we don't have anything pressed in this category
      m_fontFilter |= FNT_SERIF;
      ErrBadAttributeFilter();
      }
   else
      {
      m_viewSelectMode = FALSE;  // Only switch off VS mode if changed.
      FilterFonts();
      }
   return;
}

void CFViewerView::OnUpdateButtonSerif(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(FNTFLG_TEST(FNT_SERIF));
   return;
}


void CFViewerView::OnButtonViewselect() 
{
   m_viewSelectMode = !m_viewSelectMode;
   FilterFonts();
   return;
}

void CFViewerView::OnUpdateButtonViewselect(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(m_viewSelectMode);
   return;
}

void CFViewerView::OnViewSelected() 
{
   m_viewSelectMode = !m_viewSelectMode;
   FilterFonts();
   return;
}

//
// For the pull down/pop-up menus
//
void CFViewerView::OnUpdateViewSelected(CCmdUI* pCmdUI) 
{
   pCmdUI->SetCheck(m_viewSelectMode);
   return;
}


//---------------------------------------------------------------------
//
// Toolbar List Box/Combo Box handlers
//
//---------------------------------------------------------------------
//
// OnToolStringBoxList()
//
// This function is called when the user selects an entry from the list
// box of the string combo box on the toolbar. See FVToolBar.cpp for info.
//
//---------------------------------------------------------------------
void CFViewerView::OnToolStringBoxList() 
{
   CMainFrame* mainWnd = (CMainFrame*)AfxGetMainWnd();
   int index = mainWnd->m_wndToolBar.m_stringBox.GetCurSel();  // Selected one

   if (index < 0)	// If invalid index, we need to use whatever was current
	   {			//  which we can assume was entry 0.
	   mainWnd->m_wndToolBar.m_stringBox.DeleteString(0);
	   mainWnd->m_wndToolBar.m_stringBox.InsertString(0, testString);
	   mainWnd->m_wndToolBar.m_stringBox.SetCurSel(0);
	   }
	else
	   {
	                 // Get actual string from our array
	   testString = mainWnd->m_wndToolBar.m_stringBoxStrings[index];

						 // Replace potentially abbreviated box string with ours
	   mainWnd->m_wndToolBar.m_stringBox.DeleteString(index);
	   mainWnd->m_wndToolBar.m_stringBox.InsertString(index, testString);

						 // Set it as current selection
	   mainWnd->m_wndToolBar.m_stringBox.SetCurSel(index);

	   Invalidate();     // Update our font display
	   }

   return;
}

//---------------------------------------------------------------------
//
// OnToolStringBoxDropDown()
//
// This function is called when the user has requested that the list box
// of the string combo box be made visible.
//
//---------------------------------------------------------------------
void CFViewerView::OnToolStringBoxDropDown() 
{
   CString     str;
   CMainFrame* mainWnd = (CMainFrame*)AfxGetMainWnd();
   int index = mainWnd->m_wndToolBar.m_stringBox.GetCurSel();

   for (int i = 0; i < mainWnd->m_wndToolBar.m_stringBox.GetCount(); i++)
      {
                     // Get string from our list
      str = mainWnd->m_wndToolBar.m_stringBoxStrings[i];

      //
      // Next, let's see if we need to truncate the list box string 
      // and append "..."
      //
      if (str.GetLength() > MAX_LIST_STRING)
         {
         str = str.Left(MAX_LIST_STRING);
         str += "...";
         mainWnd->m_wndToolBar.m_stringBox.DeleteString(i);
                     // Replace string in string box
         mainWnd->m_wndToolBar.m_stringBox.InsertString(i, str);
         }
      }

   if (index >= 0)		// Only do if valid index
      {
                     // Get from our array & display in edit box
      str = mainWnd->m_wndToolBar.m_stringBoxStrings[index];
      mainWnd->m_wndToolBar.m_stringBox.SetWindowText(str);
      mainWnd->m_wndToolBar.m_stringBox.SetEditSel(0, -1);
      }
   return;
}

//---------------------------------------------------------------------
//
// OnToolStringBoxSetFocus()
//
// This function is called when the user has given focus to the edit
// portion of the string combo box.
//
//---------------------------------------------------------------------
void CFViewerView::OnToolStringBoxSetFocus() 
{
   CMainFrame* mainWnd = (CMainFrame*)AfxGetMainWnd();

   int index = mainWnd->m_wndToolBar.m_stringBox.GetCurSel();

   if (index < 0)
	   return;			// Punt if invalid index

		              // Get from our array & display in edit box
   CString str = mainWnd->m_wndToolBar.m_stringBoxStrings[index];
   mainWnd->m_wndToolBar.m_stringBox.SetWindowText(str);
   mainWnd->m_wndToolBar.m_stringBox.SetEditSel(0, -1);

   return;
}

//---------------------------------------------------------------------
//
// OnToolStringBoxEdit()
//
// This function is called when the user has makes changes in the edit
// portion of the string combo box.
//
//---------------------------------------------------------------------
void CFViewerView::OnToolStringBoxEdit() 
{
   CMainFrame* mainWnd = (CMainFrame*)AfxGetMainWnd();
   CString str;

   DWORD cursorPos = mainWnd->m_wndToolBar.m_stringBox.GetEditSel();

   mainWnd->m_wndToolBar.m_stringBox.GetWindowText(str);

   //
   // If the string the user is entering is too long, we need to pare it
   // down for the list box.
   //
   mainWnd->m_wndToolBar.m_stringBoxStrings[0] = str; // Store it in our array
   testString = str; // Make it our displayed string

   //
   // Next, let's see if we need to truncate the list box string 
   // and append "..."
   //
   if (str.GetLength() > MAX_LIST_STRING)
      {
      str = str.Left(MAX_LIST_STRING);
      str += "...";
      }

                     // Make it the new string 0
   mainWnd->m_wndToolBar.m_stringBox.DeleteString(0);
   mainWnd->m_wndToolBar.m_stringBox.InsertString(0, str);
   mainWnd->m_wndToolBar.m_stringBox.SetCurSel(0);

   //
   // The following line is required in order to overcome a weird
   // problem (MFC induced?) where on the first time the edit box has a
   // character inserted via the keyboard, it disappears from the edit 
   // box (but comes here, for some reason). The line below removes the
   // selection highlight and puts the cursor at the end of the contents
   // of the edit box.
   //
   mainWnd->m_wndToolBar.m_stringBox.SetWindowText(testString);
   mainWnd->m_wndToolBar.m_stringBox.SetEditSel((cursorPos & 0xFF),
                                                (cursorPos & 0xFF));

   Invalidate();     // Force a redraw of our display.

   return;
}

//---------------------------------------------------------------------
//
// OnToolPtSizeBoxList()
//
// This function is called when the user selects an entry from the point
// size list box on the toolbar.
//
//---------------------------------------------------------------------
void CFViewerView::OnToolPtSizeBoxList() 
{
   CMainFrame* mainWnd = (CMainFrame*)AfxGetMainWnd();
   int         index = mainWnd->m_wndToolBar.m_ptSizeBox.GetCurSel();
   char        str[4];
   int         tmpSize;

   if (index < 0)
	   return;			// Punt if invalid index

   mainWnd->m_wndToolBar.m_ptSizeBox.GetLBText(index, str); // Get size string

   tmpSize = atoi(str) * 10;
   if (pointSizeTenths != tmpSize)     // Only redo list if size changed.
      {
      m_curDispPos = DISP_TOP;
      pointSizeTenths = tmpSize;

      scrollPos = 0; // Start at beginning of list now that font size changed
      Invalidate();  // Redraw all
      }

   return;
}

//---------------------------------------------------------------------
//
// OnKybdPgDn, OnKybdPgUp, OnKybdUp, OnKybdDown, OnKybdEnd, OnKybdHome
//
// These handle keyboard driven scrolling of font display.
//
//---------------------------------------------------------------------
void CFViewerView::OnKybdCtrlSpace()	// Used for making a selection 
{
   CPoint         point;
   CRect          rect;
   GetClientRect(rect);       // Let's get the dimensions of the client window
   CClientDC      dc(this);   // Get our device context

   if (GetFocus() != this)
      SetFocus();

   if (m_viewSelectMode)   // Only applies in non-view select mode
	   return;

   point.x = point.y = 0;				// Assume start of display if no highlight

   if (!m_curFontNode || !m_fontHilit)
	  ProcessDispFontPos(point, &dc);	// Try and set highlight

   if (!m_curFontNode || !m_fontHilit) // Just in case something went wrong...
      {
      MessageBox("No font currently highlighted\nMove cursor and try again.",
               "Selection Error", MB_ICONEXCLAMATION | MB_OK);
      return;           // Drop out if no current font for some reason.
      }

   rect.top = m_curMinY;   // Toggle box.
   rect.bottom = m_curMaxY;
   DrawSelect(rect, &dc);

   m_lastSelected = m_curFontNode;  // Current font is last selected.
   m_lastSelectedStyle = m_curStyleFlag;  // Set node's selected field

   if (m_curSelected)      // If currently selected...
     {
     m_curFontNode->m_selected &= ~m_curStyleFlag;   // Strip out current style flag
     m_curSelected = FALSE;
     m_DispArray[m_curDispIndex].m_selected = FALSE;
     }
   else
     {
     m_curFontNode->m_selected |= m_curStyleFlag; // OR in current style flag
     m_curSelected = TRUE;
     m_DispArray[m_curDispIndex].m_selected = TRUE;
     }

   return;
}

void CFViewerView::OnKybdShiftSpace()   // Used for making a selection list
{
   CPoint         point;
   CRect          rect;
   GetClientRect(rect);    // Let's get the dimensions of the client window
   CClientDC      dc(this);   // Get our device context

   if (GetFocus() != this)
      SetFocus();

   if (m_viewSelectMode)   // Only applies in non-view select mode
	   return;

   point.x = point.y = 0;				// Assume start of display if no highlight

   if (!m_curFontNode || !m_fontHilit)
	  ProcessDispFontPos(point, &dc);	// Try and set highlight

   if (!m_curFontNode || !m_fontHilit) // Just in case something went wrong...
      {
      MessageBox("No font currently highlighted\nMove cursor and try again.",
               "Selection Error", MB_ICONEXCLAMATION | MB_OK);
      return;           // Drop out if no current font for some reason.
      }

   if (!m_lastSelected)		// If no last selected, need to treat as a new selection
	   {
	    ClearSelected();
		OnKybdCtrlSpace();	// Fake it out with a call to function above.
		return;
	   }

   //
   // Handle SHIFT situation next. We will use the last selected font node
   // as our starting/ending point - sequence determined by that node's
   // font name relative to the font name of the current string, as the node
   // list is sorted. Note that we will only set the selected field of fonts
   // marked for display. We also have to do special start and end case processing
   // in case we are rendering multiple styles of the same font but not start
   // on the normal case or ending on bold-italic.
   //
   m_curFontNode->m_selected |= m_curStyleFlag;  // Set node's selected field
   if (strcmp(m_curFontNode->GetString(), m_lastSelected->GetString()) < 0)
	  {  // If currently selected font is before last selected, we go here...
	  SelectAscending(m_curFontNode, m_curStyleFlag, 
	                  m_lastSelected, m_lastSelectedStyle);
	  }
   else
	  {   // If currently selected font after last selected, we go here...
      SelectAscending(m_lastSelected, m_lastSelectedStyle,
	                  m_curFontNode, m_curStyleFlag);
	  }

   Invalidate();     // Let our paint routine draw all the selections for us.

   return;
}

void CFViewerView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) // Needed to access Spacebar
{
	if (nFlags & 0xA000)	// If the key is being released, or ALT modified, ignore it here
		{
		CView::OnChar(nChar, nRepCnt, nFlags);
		return;
		}

	CPoint         point;
	CRect          rect;
   CClientDC      dc(this);   // Get our device context
	GetClientRect(rect);       // Let's get the dimensions of the client window
	point.x = point.y = 0;		// Assume start of display if no highlight

   if (m_viewSelectMode)   // Only applies in non-view select mode
	   {
       CView::OnChar(nChar, nRepCnt, nFlags);
	   return;
	   }

	switch (nChar)
		{
		case ' ':			// Spacebar - used for making a selection
		   if (!m_curFontNode || !m_fontHilit)
			  ProcessDispFontPos(point, &dc);	// Try and set highlight

		   if (!m_curFontNode || !m_fontHilit) // Just in case something went wrong...
			  {
			  MessageBox("No font currently highlighted\nMove cursor and try again.",
					   "Selection Error", MB_ICONEXCLAMATION | MB_OK);
			  return;           // Drop out if no current font for some reason.
			  }

			ClearSelected();
			m_lastSelected = m_curFontNode;  // Current font is last selected.
			m_lastSelectedStyle = m_curStyleFlag;  // Set node's selected field
			m_curFontNode->m_selected |= m_curStyleFlag;  // Set node's selected field
			m_curSelected = TRUE;
			rect.top = m_curMinY;
			rect.bottom = m_curMaxY;
			DrawSelect(rect, &dc);
			break;

		default:
			CView::OnChar(nChar, nRepCnt, nFlags);
		}
	return;
}

void CFViewerView::OnKybdUp() 
{
   CPoint      point;
   CClientDC   dc(this);   // Get our device context

   if (GetFocus() != this)
      SetFocus();

   point.x = point.y = 0;				// Assume start of display if no highlight
   if (!m_curFontNode || !m_fontHilit)
	  ProcessDispFontPos(point, &dc);	// Try and set highlight
   else
	   {
	   if (m_curDispIndex <= 0)		// At top, need to scroll up.
		   {
		   scrollPos --;           // Line Up, bounded, force redraw
		   if (scrollPos < 0)
			  scrollPos = 0;
		   m_curDispPos = DISP_TOP;
		   Invalidate();
		   }
	   else
		   {
		   point.y = m_DispArray[m_curDispIndex - 1].m_startY + 1;
		   ProcessDispFontPos(point, &dc);	// Try and set highlight
		   }
	   }
   return;
}

void CFViewerView::OnKybdDown() 
{
   CPoint      point;
   CRect       rect;
   CClientDC   dc(this);   // Get our device context

   if (GetFocus() != this)
      SetFocus();

   GetClientRect(rect);

   point.x = point.y = 0;				// Assume start of display if no highlight
   if (!m_curFontNode || !m_fontHilit)
	  ProcessDispFontPos(point, &dc);	// Try and set highlight
   else
	   {
	   if (m_DispArray[m_curDispIndex + 1].m_endY > rect.bottom) // At bottom, need to scroll down.
		   {
		   scrollPos++;               // Line down, bounded, force redraw
		   if (scrollPos > maxScroll)
			  scrollPos = maxScroll;
		   m_curDispPos = DISP_BOT;
		   Invalidate();
		   }
	   else
		   {
		   point.y = m_DispArray[m_curDispIndex + 1].m_startY + 1;
		   ProcessDispFontPos(point, &dc);	// Try and set highlight
		   }
	   }
   return;
}

void CFViewerView::OnKybdEnd() // Ctrl-PgDn
{
   if (GetFocus() != this)
      SetFocus();

   scrollPos = maxScroll;  // Go to end of list.
   m_curDispPos = DISP_BOT;
   Invalidate();
   return;
}

void CFViewerView::OnKybdHome() // Ctrl-PgUp
{
   if (GetFocus() != this)
      SetFocus();

   scrollPos = 0;          // Start at beginning
   m_curDispPos = DISP_TOP;
   Invalidate();
   return;
}

void CFViewerView::OnKybdPgdn() 
{
   if (GetFocus() != this)
      SetFocus();

   scrollPos += scrollPageSize;  // Scroll down, bounded, force redraw
   m_curDispPos = DISP_TOP;
   if (scrollPos > maxScroll)
      scrollPos = maxScroll;
   Invalidate();
   return;
}

void CFViewerView::OnKybdPgup() 
{
   if (GetFocus() != this)
      SetFocus();

   scrollPos -= scrollPageSize;  // Scroll up, bounded, force redraw
   m_curDispPos = DISP_TOP;
   if (scrollPos < 0)
      scrollPos = 0;
   Invalidate();
   return;
}

//---------------------------------------------------------------------
//
// OnVScroll()
//
// Called on a WM_VSCROLL message, handles all vertical scroll bar functions.
// This function is called multiple times for a scroll event, with the last
// call in a series being marked with an nSBCode of SB_ENDSCROLL
//
//---------------------------------------------------------------------
void CFViewerView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
   switch (nSBCode)     // Based on event, adjust the scroll position
      {
      case SB_BOTTOM:
         scrollPos = maxScroll;
         break;

      case SB_LINEDOWN:
         scrollPos++;
         break;
         
      case SB_LINEUP:
         scrollPos--;
         break;

      case SB_PAGEDOWN:
         scrollPos += scrollPageSize;
         break;
         
      case SB_PAGEUP:
         scrollPos -= scrollPageSize;
         break;
         
      case SB_THUMBPOSITION:  // We aren't doing realtime updates
      case SB_THUMBTRACK:
         scrollPos = nPos;
         break;

      case SB_TOP:
         scrollPos = 0;
         break;

      case SB_ENDSCROLL:   // Last event in series, we ignore it 'cause we're real time.
         break;

      default:
         break;
      }

   m_curDispPos = DISP_SAME;
   if (scrollPos < 0)
	  {
      scrollPos = 0;
	  m_curDispPos = DISP_TOP;
	  }

   if (scrollPos > maxScroll)
	  {
      scrollPos = maxScroll;  // Tweak values within bounds
	  m_curDispPos = DISP_BOT;
	  }

   CView::OnVScroll(nSBCode, nPos, pScrollBar);   // Process for parent

   SetScrollPos(SB_VERT, scrollPos, TRUE);
   Invalidate();

   return;
}

//---------------------------------------------------------------------
//
// DoFVSetDlg()
//
// Called when the user hits F2 or selects the Display Settings option
// from an option menu. Brings up the display settings dialog. Virtually
// all of this code is just message passing to/from the dialog box.
//
//---------------------------------------------------------------------
void CFViewerView::DoFVSetDlg() 
{
   char     str[4];
   int	   index;
   int	   oPtSize;
   UINT     oFontFilter;
   CString  oString;

   oString = testString;
   oPtSize = pointSizeTenths;
   oFontFilter = m_fontFilter;	// Backup old values

   _itoa(pointSizeTenths / 10, str, 10); // Init the dialog values
   m_SetDlg.m_ptSizeString = str;
   m_SetDlg.m_string       = testString;
   m_SetDlg.m_chkPlain     = FNTFLG_TEST(FNT_PLAIN);
   m_SetDlg.m_chkBold      = FNTFLG_TEST(FNT_BOLD);
   m_SetDlg.m_chkItalic    = FNTFLG_TEST(FNT_ITALIC);
   m_SetDlg.m_chkBoldItal  = FNTFLG_TEST(FNT_BOLDITAL);
   m_SetDlg.m_chkTrueType  = FNTFLG_TEST(FNT_TRUETYPE);
   m_SetDlg.m_chkBitmap    = FNTFLG_TEST(FNT_BITMAP);
   m_SetDlg.m_chkANSI      = FNTFLG_TEST(FNT_ANSI);
   m_SetDlg.m_chkNonANSI   = FNTFLG_TEST(FNT_NONANSI);
   m_SetDlg.m_chkSansSerif = FNTFLG_TEST(FNT_SANSSERIF);
   m_SetDlg.m_chkSerif     = FNTFLG_TEST(FNT_SERIF);
   m_SetDlg.m_chkModern    = FNTFLG_TEST(FNT_MODERN);
   m_SetDlg.m_chkScript    = FNTFLG_TEST(FNT_SCRIPT);
   m_SetDlg.m_chkOther     = FNTFLG_TEST(FNT_OTHER);

   if (m_SetDlg.DoModal() == IDOK)    // If return is OK, update window
      {
      //
      // Update the variables we need
      //
      pointSizeTenths   = atoi(m_SetDlg.m_ptSizeString) * 10;
      testString        = m_SetDlg.m_string;
      FNTFLG_SET(FNT_PLAIN, m_SetDlg.m_chkPlain);
      FNTFLG_SET(FNT_BOLD, m_SetDlg.m_chkBold);
      FNTFLG_SET(FNT_ITALIC, m_SetDlg.m_chkItalic);
      FNTFLG_SET(FNT_BOLDITAL, m_SetDlg.m_chkBoldItal);
      FNTFLG_SET(FNT_TRUETYPE, m_SetDlg.m_chkTrueType);
      FNTFLG_SET(FNT_BITMAP, m_SetDlg.m_chkBitmap);
      FNTFLG_SET(FNT_ANSI, m_SetDlg.m_chkANSI);
      FNTFLG_SET(FNT_NONANSI, m_SetDlg.m_chkNonANSI);
      FNTFLG_SET(FNT_SANSSERIF, m_SetDlg.m_chkSansSerif);
      FNTFLG_SET(FNT_SERIF, m_SetDlg.m_chkSerif);
      FNTFLG_SET(FNT_MODERN, m_SetDlg.m_chkModern);
      FNTFLG_SET(FNT_SCRIPT, m_SetDlg.m_chkScript);
      FNTFLG_SET(FNT_OTHER, m_SetDlg.m_chkOther);

      //
      // Also need to update current list box selections
      //
      CMainFrame* mainWnd = (CMainFrame*)AfxGetMainWnd();
      mainWnd->m_wndToolBar.m_stringBox.DeleteString(0);
      mainWnd->m_wndToolBar.m_stringBox.InsertString(0, testString);
      mainWnd->m_wndToolBar.m_stringBox.SetCurSel(0);
      mainWnd->m_wndToolBar.m_stringBoxStrings[0] = testString;
      _itoa(pointSizeTenths / 10, str, 10); // Init the dialog values
      index = mainWnd->m_wndToolBar.m_ptSizeBox.FindStringExact(0, str);
	   if (index != CB_ERR)
         mainWnd->m_wndToolBar.m_ptSizeBox.SetCurSel(index);  // Set point size list box

	   if (oPtSize != pointSizeTenths || oFontFilter != m_fontFilter || oString != testString)
         {
         m_viewSelectMode = FALSE;    // Turn off view selected mode, just in case.
         FilterFonts();    // Only reset font display if point size or other info changed
         Invalidate();        // Invalidate the whole window, force redraw
         }
      }

   return;
}

//---------------------------------------------------------------------
//
// DrawBox(CRect rect, BOOL erase)
//
// Draws an XOR outline box in the desired location. The erase flag is
// currently ignored.
//
//---------------------------------------------------------------------
void CFViewerView::DrawBox(CRect rect, BOOL erase, CDC *pDC)
{
   CPen*       pPen;
   int         oldROP = pDC->GetROP2();   // Old ROP
   int         oldBkMode = pDC->GetBkMode(); // Save old background mode
   
   pPen = new CPen(PS_DOT, 1, RGB(255, 255, 255)); // XOR with all bits on
   CPen* pOldPen = pDC->SelectObject(pPen);

   pDC->SetROP2(R2_XORPEN);      // Set to an XOR ROP
   pDC->SetBkMode(TRANSPARENT);  // Make bk portion of dotted line invis.

   pDC->MoveTo(rect.left, rect.top);
   pDC->LineTo(rect.right - 1, rect.top);
   pDC->LineTo(rect.right - 1, rect.bottom);
   pDC->LineTo(rect.left, rect.bottom);
   pDC->LineTo(rect.left, rect.top);

   pDC->SetBkMode(oldBkMode);
   pDC->SetROP2(oldROP);         // Restore the previous ROP & BkMode
   pDC->SelectObject(pOldPen);

   delete pPen;						// Delete our created object
   return;
}

//---------------------------------------------------------------------
//
// DrawSelect(CRect rect, BOOL erase)
//
// Draws an XOR selection rectangle at the desired location. The erase
// flag is currently ignored.
//
//---------------------------------------------------------------------
void CFViewerView::DrawSelect(CRect rect, CDC *pDC)
{
   DrawBox(rect, TRUE, pDC);

   pDC->InvertRect(rect);

   DrawBox(rect, FALSE, pDC);

   return;
}

//---------------------------------------------------------------------
//
// ClearSelected()
//
// This function parses through all the displayed fonts and clears
// the selection flags on all those that have been previously selected.
// Called only when the user does an unmodified click in the font window.
//
//---------------------------------------------------------------------
void CFViewerView::ClearSelected()
{
   CFViewerDoc*  pDoc = GetDocument();
   ASSERT_VALID(pDoc);
   CFontNode      *pNode = pDoc->faceList.GetStart(); // Start of face list

   m_curDispPos = DISP_SAME;

   while (pNode != NULL)  // While not at end of list...
      {
      pNode->m_selected = 0;     // Indicate that it's not selected
      pNode = pNode->GetNext();
      }

   m_lastSelected = NULL; // No fonts currently selected
   Invalidate();  // Redraw and clear the selected flag for displayed fonts
   return;
}

//---------------------------------------------------------------------
//
// SelectAscending(start, end)
//
// This function sets the m_selected field in each FontNode from startNode
// to endNode, based on the currently selected rendering styles. Special
// attention is paid to the start and end nodes themselves, because if more
// than a single rendering style is active, it's quite possible that the entire
// font may not be selected. Instead, only a subset of the font renderings may
// be. We only set the selected fields on nodes that have the display bit
// enabled.
//
// Note that we assume that both nodes have their m_selected fields set to 
// some valid value, and that both are displayed.
//
//---------------------------------------------------------------------
void  CFViewerView::SelectAscending(CFontNode* startNode, int startStyle,
                              CFontNode* endNode, int endStyle)
{
   CFontNode*  ptr;
   int         i;

   //
   // First, let's deal with the start node. We need to check for the special
   // case where the endNode has the same name, and therefore specifies the same
   // font or font family (as in ANSI vs. non-ANSI faces, or even just renderings
   // of the same font).
   //
   if (strcmp(startNode->GetString(), endNode->GetString()) == 0)
      {
      //
      // If we get here, we're dealing with the same font name and need to
      // do some additional work in figuring out what to select. There are two
      // cases in which this occurs: the nodes are the same, and we're
      // asked to select all style renderings inclusive of the ones specified.
      // The other case is when the non-ANSI setting is enabled and we
      // have multiple variations of the same font.
      // The first case is easy to deal with because we can actually compare
      // the start and end nodes and the matching styles to see which ones to
      // fill in.
      // The second case is a little tougher, because there isn't necessarily
      // a predetermined order of font subsets. In that case, we start at the
      // beginning of the node list and search until we find a name match,
      // and then iterate until we find either the start or end node, make that
      // the start node, the other the end node, and then continue on with the 
      // rest of the code.
      //
      if (startNode == endNode)
         {
         //
         // Need to sort styles.
         //
         if (startStyle > endStyle)
            {     // Swap start and end styles
            i = endStyle;
            endStyle = startStyle;
            startStyle = i;
            }
         for (i = 0; i < m_renderVariations; i++)
            {  // OR in any styles between start and end
            if ((startStyle < m_rendVarTypes[i]) && (endStyle > m_rendVarTypes[i]))
               startNode->m_selected |= m_rendVarTypes[i];  // OR this type in
            }
         return;     // We're done here
         }

      //
      // Must be same font names, different character sets.
      //
      CFViewerDoc*  pDoc = GetDocument();
      ASSERT_VALID(pDoc);

      ptr = pDoc->faceList.GetStart(); // Start of face list
      //
      // Loop while we're not at end of list and until we find a name match.
      while (ptr != NULL && (strcmp(startNode->GetString(), ptr->GetString())))
         ptr = ptr->GetNext();

      while (ptr != NULL)        // Now try and find start or end nodes
         {
         if (ptr == startNode)   // If we found start first, okay to continue on.
            break;
         if (ptr == endNode)     // If we found end first, swap nodes and continue.
            {
            ptr = endNode;
            endNode = startNode;
            startNode = ptr;
            i = endStyle;
            endStyle = startStyle;
            startStyle = i;
            break;
            }
         ptr = ptr->GetNext();
         }
      }

   //
   // Next, since we are dealing with different fonts, let's handle the 
   // start node case of where we may need to update the m_selected field
   // to include other font rendering styles beyond the start style.
   // To do that, we determine the maximum render level of the start style,
   // and then compare that to the number of currently valid render 
   // variations. Note that the m_rendVarTypes array contains only
   // the rendering styles currently selected, the number of which is 
   // determined by m_renderVariations (counting from 1).
   //
   for (i = m_renderVariations - 1; i >= 0; --i)
      {
      if (startStyle & m_rendVarTypes[i])       // If font style is set
         break;                           //  we're done here.
      startNode->m_selected |= m_rendVarTypes[i];  // OR this type in
      }
   //
   // Then, until we hit the end node, let's set the selected field for each node
   //
   ptr = startNode->GetNext();
   while (ptr != NULL && ptr != endNode)     // While not at end of list, or endNode...
      {
      if (ptr->m_faceFlag & FNT_DISPLAY)
          ptr->m_selected = m_rendCompound;  // Indicate it's all selected.
      ptr = ptr->GetNext();
      }

   //
   // Finally, we have to do the reverse of the start case, and only select those
   // those styles that preceed the end style.
   for (i = 0; i < m_renderVariations; i++)
      {
      if (endStyle & m_rendVarTypes[i])         // If font style is set
         break;                           //  we're done here.
      endNode->m_selected |= m_rendVarTypes[i]; // OR this type in
      }

   return;
}


//---------------------------------------------------------------------
//
// DrawVSplitter(int x)
//
// Draws a vertical splitter bar centered around the specified X location.
// Splitter is drawn with a highlighted left edge, normal middle (3 pixels),
// shadow right edge.
//
//---------------------------------------------------------------------
void CFViewerView::DrawVSplitter(int splitX, CDC *pDC)
{
   CRect       rect;
   int         x, height;
   
   GetClientRect(rect);    // Let's get the dimensions of the client window
   x = rect.left + splitX;
   height = rect.bottom - rect.top + 1;

   // 
   // Let's draw the left edge first.
   //
   pDC->FillSolidRect(x - 2, rect.top, 1, height, GetSysColor(COLOR_3DHILIGHT));
   pDC->FillSolidRect(x - 1, rect.top, 3, height, GetSysColor(COLOR_3DFACE));
   pDC->FillSolidRect(x + 2, rect.top, 1, height, GetSysColor(COLOR_3DDKSHADOW));

   return;
}

//---------------------------------------------------------------------
//
// OnLButtonDown
//
// We use this function to trap requests for selecting/deselecting a 
// currently displayed font.
//
// If we're in the splitter zone, we ignore this request, as the splitter has
// visual priority.
// 
// If we're safe (i.e. not in the splitter zone), we process the down-click
// in the following fashion:
//
// 1) If the CTRL or SHIFT keys aren't being pressed at the same time,
//    we clear all other selections currently active, and select the current
//    font. We also note this as the last font selected.
// 2) If the CTRL key is being pressed, we invert the selection state of the
//    current font. In other words, if it's not selected, we add it to the
//    selection list. If it was selected, we deselect it, and remove it from
//    the selection list. We also note this as the last font selected.
// 3) If the SHIFT key is being pressed (takes priority over CTRL), we select
//    all fonts between the last selected font and the current one.
//
// nFlag values we're interested in:
//    MK_CONTROLSet if the CTRL key is down.
//    MK_SHIFTSet if the SHIFT key is down.
//
//---------------------------------------------------------------------
void CFViewerView::OnLButtonDown(UINT nFlags, CPoint point) 
{
   CRect       rect;
   CClientDC   dc(this);   // Get our device context

   if (GetFocus() != this)
      {
      CView::OnLButtonDown(nFlags, point);
      return;
      }

   GetClientRect(rect);    // Let's get the dimensions of the client window

   //
   // Punt if on top of splitter bar. Handled in OnMouseMove().
   //
   if (point.x >= m_splitX - 4 && point.x <= m_splitX + 4)
      {
      CView::OnLButtonDown(nFlags, point);
      return;
      }

   if (m_viewSelectMode)
      {
      m_viewSelectMode = FALSE;  // Turn it off if it's on.
      FilterFonts();          // Redo the font filter
      CView::OnLButtonDown(nFlags, point);
      return;
      }

   ProcessDispFontPos(point, &dc);	// Process the current point for an outline
                                // and focus information

   if (!m_curFontNode || !m_fontHilit) // Just in case something went wrong...
      {
      MessageBox("No font currently highlighted\nMove cursor and try again.",
               "Selection Error", MB_ICONEXCLAMATION | MB_OK);
      CView::OnLButtonDown(nFlags, point);
      return;           // Drop out if no current font for some reason.
      }

   //
   // Handle case where an un-modified click is received, or if a shift click
   // is received with no last selection present.
   //
   if (!((nFlags & MK_CONTROL) || (nFlags & MK_SHIFT))
      || !m_lastSelected && nFlags & MK_SHIFT)
      {
      ClearSelected();
      m_lastSelected = m_curFontNode;  // Current font is last selected.
      m_lastSelectedStyle = m_curStyleFlag;  // Set node's selected field
      m_curFontNode->m_selected |= m_curStyleFlag;  // Set node's selected field
      m_curSelected = TRUE;
      rect.top = m_curMinY;
      rect.bottom = m_curMaxY;
      DrawSelect(rect, &dc);
      CView::OnLButtonDown(nFlags, point);
      return;
      }

   //
   // Handle SHIFT situation next. We will use the last selected font node
   // as our starting/ending point - sequence determined by that node's
   // font name relative to the font name of the current string, as the node
   // list is sorted. Note that we will only set the selected field of fonts
   // marked for display. We also have to do special start and end case processing
   // in case we are rendering multiple styles of the same font but not start
   // on the normal case or ending on bold-italic.
   //
   if (nFlags & MK_SHIFT)
      {
      m_curFontNode->m_selected |= m_curStyleFlag;  // Set node's selected field
      if (strcmp(m_curFontNode->GetString(), m_lastSelected->GetString()) < 0)
         {  // If currently selected font is before last selected, we go here...
         SelectAscending(m_curFontNode, m_curStyleFlag, 
                     m_lastSelected, m_lastSelectedStyle);
         }
      else
         {  // If currently selected font after last selected, we go here...
         SelectAscending(m_lastSelected, m_lastSelectedStyle,
                     m_curFontNode, m_curStyleFlag);
         }
      Invalidate();     // Let our paint routine draw all the selections for us.
      CView::OnLButtonDown(nFlags, point);
      return;
      }

   //
   // Handle CTRL situation last. This toggles the current select state, sets
   // the last selection to current.
   //
   if (nFlags & MK_CONTROL)
      {
      rect.top = m_curMinY;   // Toggle box.
      rect.bottom = m_curMaxY;
      DrawSelect(rect, &dc);

      m_lastSelected = m_curFontNode;  // Current font is last selected.
      m_lastSelectedStyle = m_curStyleFlag;  // Set node's selected field

      if (m_curSelected)      // If currently selected...
         {
         m_curFontNode->m_selected &= ~m_curStyleFlag;   // Strip out current style flag
         m_curSelected = FALSE;
         m_DispArray[m_curDispIndex].m_selected = FALSE;
         }
      else
         {
         m_curFontNode->m_selected |= m_curStyleFlag; // OR in current style flag
         m_curSelected = TRUE;
         m_DispArray[m_curDispIndex].m_selected = TRUE;
         }
      }

   CView::OnLButtonDown(nFlags, point);
   return;
}

//---------------------------------------------------------------------
//
// OnLButtonDblClick()
//
// This gets called whenever Windows gets a double click on the left
// mouse button for us. Since we don't do anything special for double
// clicks, we instead pass a call for this function to the OnLButtonDown()
// function above, since we want it to respond on all clicks, single
// or double.
//
//---------------------------------------------------------------------
void CFViewerView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	OnLButtonDown(nFlags, point);
	CView::OnLButtonDblClk(nFlags, point);
}

//---------------------------------------------------------------------
//
// OnSetCursor()
//
// This gets called whenever Windows thinks our window needs to have the
// mouse cursor updated. We intercept this function so we can have a nice
// and smooth, non-flickering cursor when we're moving the splitter bar.
//
//---------------------------------------------------------------------
BOOL CFViewerView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message)
{
   if (nHitTest == HTCLIENT && m_splitCursor && GetFocus() == this)   // If in client area & splitting
      {
      SetCursor(m_hCursorSizeWE);               // Set splitter cursor
      return(TRUE);
      }
   else
      return(CView::OnSetCursor(pWnd, nHitTest, message)); // Punt
}

//---------------------------------------------------------------------
//
// OnKillFocus()
//
// This gets called whenever our font window is about to loose focus
// so we can remove our highlight box. - No longer really need after
// 1.09E.
//
//---------------------------------------------------------------------
void CFViewerView::OnKillFocus(CWnd* pNewWnd) 
{
   CView::OnKillFocus(pNewWnd);

   return;
}

//---------------------------------------------------------------------
//
// ProcessDispFontPos(CPoint point)
//
// This gets called whenever we want to update the current font highlight position & info
//
//---------------------------------------------------------------------
void CFViewerView::ProcessDispFontPos(CPoint point, CDC *pDC)
{
   CRect rect;    // Client rectangle
   CRect oldRect;
   SIZE  drwWidth;
   int   dispIndex = 0; // Our loop and font

   GetClientRect(rect); // Let's get the dimensions of the client window

   drwWidth.cx = drwWidth.cy = 1;   // Set our rectangle width
                     // We need the left and right coordinates only.
   if (m_fontHilit)
      {
      // If there's an old rectangle, set its coordinates
      oldRect.top = m_curMinY;
      oldRect.bottom = m_curMaxY;
      oldRect.right = rect.right;
      oldRect.left = rect.left;
      }

                     // Parse through font table list until matching Y range found
   while (m_DispArray[dispIndex].m_endY < point.y)
      dispIndex++;

   rect.top = m_DispArray[dispIndex].m_startY;  // Set rect to new Y range
   rect.bottom = m_DispArray[dispIndex].m_endY;

   if (rect.top <= point.y && point.y <= rect.bottom) // Double check for correctness
      {
      if (m_fontHilit)     // If there currently is a highlight box in place, remove it
         DrawBox(oldRect, TRUE, pDC);

      DrawBox(rect, FALSE, pDC);   // Draw the new one

      // Set our variables for future matches
      m_fontHilit = TRUE;
      m_curMinY = rect.top;
      m_curMaxY = rect.bottom;
      m_curFontNode = m_DispArray[dispIndex].m_fontNode;
      m_curFullName = m_DispArray[dispIndex].m_fullName;
      m_curStyleFlag = m_DispArray[dispIndex].m_styleFlag;
      m_curSelected = m_DispArray[dispIndex].m_selected;
      m_curDispIndex = dispIndex;
      switch (m_curStyleFlag)
         {
         case FNT_PLAIN:
            m_curFontNode->m_logFont.lfWeight = FW_NORMAL;
            m_curFontNode->m_logFont.lfItalic = FALSE;
            break;
         case FNT_BOLD:
            m_curFontNode->m_logFont.lfWeight = FW_BOLD;
            m_curFontNode->m_logFont.lfItalic = FALSE;
            break;
         case FNT_ITALIC:
            m_curFontNode->m_logFont.lfWeight = FW_NORMAL;
            m_curFontNode->m_logFont.lfItalic = TRUE;
            break;
         case FNT_BOLDITAL:
            m_curFontNode->m_logFont.lfWeight = FW_BOLD;
            m_curFontNode->m_logFont.lfItalic = TRUE;
            break;
         }

      CMainFrame* mainWnd = (CMainFrame*)AfxGetMainWnd();
      mainWnd->m_wndStatusBar.SetPaneText(0, m_curFullName, TRUE);
      }
	return;
}


//---------------------------------------------------------------------
//
// OnMouseMove
//
// We use this function to determine where the mouse cursor currently is.
// The X position is used to see if we're over the splitter bar, in which
// case we change the cursor type to a double headed horizontal set of
// arrows and leave, unless the left button is down, in which case we
// move the splitter bar.
//
// Otherwise, we look at the Y position, which is used to determine where
// to put the light gray box which highlights the current font the mouse 
// cursor is on or near.
//
// First, we determine if the mouse has moved off the most recently 
// highlighted font. If not, we exit. If it has, we have to determine
// which font the mouse is currently on.
//
// In order to quickly make this determination, we generated a table of
// Y start and Y end values with links to the font name at each Y start.
// This table is then quickly parsed to see which font the mouse is over.
// The table is actually a CArray of CDisplayedFont classes, referenced by
// m_DispArray.
//
// When the Y mouse position switches to a new font, we erase the old 
// highlight box, and replace it with a new one around the newly 
// selected font.
//
//---------------------------------------------------------------------
void CFViewerView::OnMouseMove(UINT nFlags, CPoint point) 
{
   CRect rect;    // Client rectangle

   if (GetFocus() != this)    // Ignore requests if not our focus.
      {
      CView::OnMouseMove(nFlags, point);
      return;
      }

   GetClientRect(rect); // Let's get the dimensions of the client window

   //
   // First, let's see if the left mouse button is currently pressed.
   // If so, let's see if tracking is on, in which case we are moving the
   // splitter bar, and will continue doing so until the button is no longer
   // being pressed.
   //
   if (nFlags & MK_LBUTTON)
      {
      // Button is down.
      if (m_splitTracking)          // Are we tracking?
         {
         SetCursor(m_hCursorSizeWE);      // Yes. Set the proper cursor.
         m_splitCursor = TRUE;
         int deltaX = point.x - m_splitTrackLastX;  // Figure out how far we moved

         if (deltaX)    // Only bother updating things if we actually moved
            {
            int oldSplitX = m_splitX; // Let's help avoid some flicker

            m_splitX += deltaX;  // Move splitter
            if (m_splitX < rect.left + 50)   // But bound it so it can't go too far
               m_splitX = rect.left + 50;
            if (m_splitX > rect.right - 50)
               m_splitX = rect.right - 50;

            if (m_splitX != oldSplitX) // Check if really changed.
               {
               m_splitTrackLastX = point.x;
               Invalidate();     // Let's redraw to show new splitter!
               }
            }
         return;        // Nothing else to be done here.
         }
      }
   else
      {
      // Button is not down, let's make sure our tracking is cleared.
      m_splitTracking = FALSE;
      m_splitCursor = FALSE;
      }

   //
   // Next, check to see if the cursor is above our splitter bar
   //
   if (point.x >= m_splitX - 4 && point.x <= m_splitX + 4)
      {
      // If we get here, we are on the splitter bar, so we need to change
      // the cursor to the right kind.

      // Load the predefined Windows "size left/right" cursor.
      SetCursor(m_hCursorSizeWE);
      m_splitCursor = TRUE;

      //
      // Now, if the left mouse button is down, then we need to enable tracking.
      // The actual tracking will be done next time this function is called, via
      // the code at front.
      //
      if (nFlags & MK_LBUTTON)
         {
         m_splitTracking = TRUE;
         m_splitTrackLastX = point.x;  // Set the tracking point.
         }

      return;     // Nothing left to do.
      }
   
   CView::OnMouseMove(nFlags, point);
   return;
}

//---------------------------------------------------------------------
//
// OnShowCharSet()
//
// When this option is selected, we need to pop-up a dialog box containing
// a text window with the complete rendering of all characters in the font
//  selected, in a grid.
//
//---------------------------------------------------------------------
void CFViewerView::OnShowCharSet() 
{
   if (!m_curFontNode)
      {
      MessageBox("No font currently selected or highlighted!", "Display Error",
               MB_ICONEXCLAMATION | MB_OK);
      return;           // Drop out if no current font for some reason.
      }
   m_DispFntDlg.m_fontNode = m_curFontNode;  // Send Font info to Dialog
   m_DispFntDlg.m_fullName = m_curFullName;
   m_DispFntDlg.m_typeOfShow = TRUE;         // Select grid display
   m_DispFntDlg.DoModal();    // Display the font

   Invalidate();        // Force redraw of screen
   return;
}

//---------------------------------------------------------------------
//
// OnShowAlphaNum()
//
// When this option is selected, we need to pop-up a dialog box containing 
// a text window with the complete rendering of all the regular text and
// numeric characters in the font selected.
//
//---------------------------------------------------------------------
void CFViewerView::OnShowAlphaNum() 
{
   if (!m_curFontNode)
      {
      MessageBox("No font currently selected or highlighted!", "Display Error",
               MB_ICONEXCLAMATION | MB_OK);
      return;           // Drop out if no current font for some reason.
      }
   m_DispFntDlg.m_fontNode = m_curFontNode;  // Send Font info to Dialog
   m_DispFntDlg.m_fullName = m_curFullName;
   m_DispFntDlg.m_typeOfShow = FALSE;        // Select AlphaNum display
   m_DispFntDlg.DoModal();    // Display the font

   Invalidate();        // Force redraw of screen
   return;
}

//---------------------------------------------------------------------
//
// OnRButtonDown()
//
// This function pops up our right click menu, which is just a copy of
// the Options pull-down.
//
//---------------------------------------------------------------------
void CFViewerView::OnRButtonDown(UINT nFlags, CPoint mousePoint) 
{
   CClientDC   dc(this);   // Get our device context

   if (GetFocus() != this)
      {
      CView::OnRButtonDown(nFlags, mousePoint);
      return;
      }

   //CMainFrame* mainWnd = (CMainFrame*)AfxGetMainWnd();

   Invalidate();
   UpdateWindow();
   ProcessDispFontPos(mousePoint, &dc);	// Move the focus

   CPoint point = mousePoint;
   CMenu menu;
   VERIFY(menu.LoadMenu(IDR_POPMENU));
   CMenu* pPopup = menu.GetSubMenu(0);
   ASSERT(pPopup != NULL);

   ClientToScreen(&point);
   pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, AfxGetMainWnd());
   
   CView::OnRButtonDown(nFlags, point);
	Invalidate();
}

//---------------------------------------------------------------------
// CDisplayedFont() - Constructor
//---------------------------------------------------------------------
CDisplayedFont::CDisplayedFont()
{
   m_startY = m_endY = 0;
   m_styleFlag = 0;     // Specifies Normal, Bold, Italic
   m_fullName = "";
   m_selected = FALSE;        // Not currently selected
   m_fontNode = NULL;
   return;
}

/////////////////////////////////////////////////////////////////////////////
// CFViewerView diagnostics

#ifdef _DEBUG
void CFViewerView::AssertValid() const
{
   CView::AssertValid();
}

void CFViewerView::Dump(CDumpContext& dc) const
{
   CView::Dump(dc);
}

CFViewerDoc* CFViewerView::GetDocument() // non-debug version is inline
{
   ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CFViewerDoc)));
   return (CFViewerDoc*)m_pDocument;
}
#endif //_DEBUG

